##### Install utility functions #####

function(OBLO_PRETTY_LIST_PRINT OBLO_LIST)
    foreach(LIST_ITEM IN LISTS OBLO_LIST)
        message(STATUS "   ${LIST_ITEM}")
    endforeach()
endfunction()

function (OBLO_CONFIGURE_FILES OBLO_DIR)
    file(GLOB_RECURSE
            OBLO_TEMPLATE_FILES
            "${OBLO_DIR}/*.template")
    foreach(OBLO_INPUT_FILE ${OBLO_TEMPLATE_FILES})
        string(REPLACE
                ".template"
                "" OBLO_OUTPUT_FILE "${OBLO_INPUT_FILE}")
        configure_file(${OBLO_INPUT_FILE} ${OBLO_OUTPUT_FILE})
        message(STATUS "Configuration COMPLETED")
        message(STATUS "     input: ${OBLO_INPUT_FILE}")
        message(STATUS "     output: ${OBLO_OUTPUT_FILE}")
    endforeach()
endfunction()

function(OBLO_MERGE_FILES OBLO_IN_DIR OBLO_OUT_DIR)
    file(GLOB_RECURSE
            OBLO_PART_FILES
            "${OBLO_IN_DIR}/*.part")
    foreach(OBLO_INPUT_FILE ${OBLO_PART_FILES})
        string(REGEX REPLACE
                "${OBLO_IN_DIR}/(.*)\\.part"
                "${OBLO_OUT_DIR}/\\1" OBLO_OUTPUT_FILE "${OBLO_INPUT_FILE}")
        if(NOT EXISTS ${OBLO_OUTPUT_FILE})
            file(WRITE ${OBLO_OUTPUT_FILE} "")
        endif()
        file(READ ${OBLO_INPUT_FILE} OBLO_INPUT_FILE_CONTENTS)
        file(APPEND ${OBLO_OUTPUT_FILE} "${OBLO_INPUT_FILE_CONTENTS}")
        message(STATUS "Append COMPLETED")
        message(STATUS "     input: ${OBLO_INPUT_FILE}")
        message(STATUS "     output: ${OBLO_OUTPUT_FILE}")
    endforeach()
endfunction()

function(OBLO_REPLACE_FILES OBLO_IN_DIR OBLO_OUT_DIR)
    if(EXISTS "${OBLO_IN_DIR}")
        message(STATUS "Replacing data from ${OBLO_IN_DIR}...")
        file(GLOB_RECURSE
                OBLO_REPLACE_FILES
                "${OBLO_IN_DIR}/*")
        foreach(OBLO_INPUT_FILE ${OBLO_REPLACE_FILES})
            string(REGEX REPLACE
                    "${OBLO_IN_DIR}"
                    "${OBLO_OUT_DIR}" OBLO_OUTPUT_FILE "${OBLO_INPUT_FILE}")
            list(APPEND OBLO_REPLACE_FILES_RMV "${OBLO_OUTPUT_FILE}")
        endforeach()
        if(OBLO_REPLACE_FILES_RMV)
            file(REMOVE ${OBLO_REPLACE_FILES_RMV})
        endif()
        file(COPY "${OBLO_IN_DIR}/" DESTINATION "${OBLO_OUT_DIR}")
        message(STATUS "data REPLACED in ${OBLO_OUT_DIR}.")
    endif()
endfunction()

function(OBLO_LIST_SUBDIRS OBLO_ROOTDIR OBLO_OUTPUT_LIST)
  file(GLOB OBLO_SUBDIRS RELATIVE "${OBLO_ROOTDIR}" "${OBLO_ROOTDIR}/*")
  set(OBLO_LIST_SUBDIRS_RESULT "")

  foreach(OBLO_SUBDIR ${OBLO_SUBDIRS})
    if(IS_DIRECTORY ${OBLO_ROOTDIR}/${OBLO_SUBDIR})
      list(APPEND OBLO_LIST_SUBDIRS_RESULT ${OBLO_SUBDIR})
    endif()
  endforeach()

  list(SORT OBLO_LIST_SUBDIRS_RESULT)
  set(${OBLO_OUTPUT_LIST} ${OBLO_LIST_SUBDIRS_RESULT} PARENT_SCOPE)
endfunction()

function(OBLO_GENERATE_INSTALL_FILES OBLO_STAGING_DIR OBLO_PRODUCT_DIR PLATFORM CUSTOMER)
    ##### Clean tmp dir #####
    message(STATUS "OBLO_STAGING_DIR: ${OBLO_STAGING_DIR}")
    file(REMOVE_RECURSE "${OBLO_STAGING_DIR}/")

    ##### Install original data #####
    file(COPY "${OBLO_PRODUCT_DIR}/base/cfg/"
            DESTINATION "${OBLO_STAGING_DIR}/cfg")
    file(COPY "${OBLO_PRODUCT_DIR}/base/data/"
            DESTINATION "${OBLO_STAGING_DIR}/data")

    if(EXISTS "${OBLO_PRODUCT_DIR}/platform/${PLATFORM}/cfg/")
        file(COPY "${OBLO_PRODUCT_DIR}/platform/${PLATFORM}/cfg/"
                DESTINATION "${OBLO_STAGING_DIR}/platform/cfg")
    endif()
    if(EXISTS "${OBLO_PRODUCT_DIR}/platform/${PLATFORM}/data/")
        file(COPY "${OBLO_PRODUCT_DIR}/platform/${PLATFORM}/data/"
                DESTINATION "${OBLO_STAGING_DIR}/platform/data")
    endif()

    if(EXISTS "${OBLO_PRODUCT_DIR}/customer/${CUSTOMER}/cfg/")
        file(COPY "${OBLO_PRODUCT_DIR}/customer/${CUSTOMER}/cfg/"
                DESTINATION "${OBLO_STAGING_DIR}/customer/cfg")
    endif()
    if(EXISTS "${OBLO_PRODUCT_DIR}/customer/${CUSTOMER}/data/")
        file(COPY "${OBLO_PRODUCT_DIR}/customer/${CUSTOMER}/data/"
                DESTINATION "${OBLO_STAGING_DIR}/customer/data")
    endif()

    ##### Configure template files with CMake variables #####
    OBLO_CONFIGURE_FILES("${OBLO_STAGING_DIR}")

    ##### Replace files #####
    OBLO_REPLACE_FILES("${OBLO_STAGING_DIR}/platform" "${OBLO_STAGING_DIR}")
    OBLO_REPLACE_FILES("${OBLO_STAGING_DIR}/customer" "${OBLO_STAGING_DIR}")

    ##### Merge part files #####
    OBLO_MERGE_FILES("${OBLO_STAGING_DIR}/platform" "${OBLO_STAGING_DIR}")
    OBLO_MERGE_FILES("${OBLO_STAGING_DIR}/customer" "${OBLO_STAGING_DIR}")

    #### Remove trash #####
    file(REMOVE_RECURSE "${OBLO_STAGING_DIR}/platform")
    file(REMOVE_RECURSE "${OBLO_STAGING_DIR}/customer")
    file(GLOB_RECURSE OBLO_PART_FILES "${OBLO_STAGING_DIR}/*.part")
    if(OBLO_PART_FILES)
        file(REMOVE ${OBLO_PART_FILES})
    endif()
    file(GLOB_RECURSE OBLO_TEMPLATE_FILES "${OBLO_STAGING_DIR}/*.template")
    if(OBLO_TEMPLATE_FILES)
        file(REMOVE ${OBLO_TEMPLATE_FILES})
    endif()
    message(STATUS "OBLO_STAGING_DIR: ${OBLO_STAGING_DIR}")
endfunction()

function(OBLO_COPY_DIR OBLO_IN_DIR OBLO_OUT_DIR)
    message(STATUS "Copying directory")
    message(STATUS "   from : ${OBLO_IN_DIR}")
    message(STATUS "   to   : ${OBLO_OUT_DIR}")
    execute_process(
            COMMAND ${CMAKE_COMMAND} -E copy_directory
            "${OBLO_IN_DIR}"
            "${OBLO_OUT_DIR}")
    message(STATUS "copying COMPLETED.")
endfunction()

