#include "binder.h"
#include <stdlib.h>
#include <string.h>

extern int run;

static broker_binder_t* mosq_binder;

void broker_set_running(int running) {
    run = running;
}

void broker_set_binder(broker_binder_t* mosq_broker_binder) {
    mosq_binder = mosq_broker_binder;
}

void broker_on_log_print(const char* log, broker_log_level_t level) {
    if ( mosq_binder && mosq_binder->on_log_print ) {
        mosq_binder->on_log_print(log, level);
    }
}

int broker_on_acl_check(const char* clientid, const char* username, const char* topic, int access) {
    if ( mosq_binder && mosq_binder->on_acl_check ) {
        return mosq_binder->on_acl_check(clientid, username, topic, access);
    }

    return BROKER_RET_SUCCESS;
}

int broker_on_unpwd_check(const char* username, const char* password, int port) {
    if ( mosq_binder && mosq_binder->on_unpwd_check ) {
        return mosq_binder->on_unpwd_check(username, password, port);
    }

    return BROKER_RET_SUCCESS;
}

void broker_on_client_connected(const char* clientid, const char* username) {
    if ( mosq_binder && mosq_binder->on_client_connected ) {
        mosq_binder->on_client_connected(clientid, username);
    }
}

void broker_on_client_disconnected(const char* clientid, const char* username) {
    if ( mosq_binder && mosq_binder->on_client_disconnected ) {
        mosq_binder->on_client_disconnected(clientid, username);
    }
}

int broker_on_client_do_disconnect_check(const char *clientid) {
    if ( mosq_binder && mosq_binder->on_client_do_disconnect_check ) {
        mosq_binder->on_client_do_disconnect_check(clientid);
    }
}

int broker_on_pem_password_required(char* buf, int size, int rwflag, void* userdata) {
    int rc = 0;

    if (!rwflag) {
        strncpy(buf, (const char*)(userdata), size);
        buf[size-1] = '\0';
        rc = (int)strlen(buf);
    }

    return rc;
}

void* broker_get_pem_password_userdata(unsigned int port) {
    void* userdata = NULL;

    if ( mosq_binder && mosq_binder->get_pem_password ) {
        userdata = (void*) mosq_binder->get_pem_password(port);
    }

    return userdata;
}
