#ifndef MOSQUITTO_OMB_BINDER
#define MOSQUITTO_OMB_BINDER

#include <stdint.h>
#include <mosquitto_plugin.h>

typedef enum {
    BROKER_LOG_TRACE = 0,
    BROKER_LOG_DEBUG,
    BROKER_LOG_NOTICE,
    BROKER_LOG_INFORMATION,
    BROKER_LOG_WARNING,
    BROKER_LOG_ERROR
} broker_log_level_t;

enum broker_ret_values {
    BROKER_RET_SUCCESS = 0,
    BROKER_RET_INVAL = 3,
    BROKER_RET_AUTH_FAILED = 11,
    BROKER_RET_ACL_DENIED = 12
};

typedef enum broker_permissions {
    BROKER_PERM_NONE = MOSQ_ACL_NONE,
    BROKER_PERM_READ = MOSQ_ACL_READ,
    BROKER_PERM_WRITE = MOSQ_ACL_WRITE,
    BROKER_PERM_SUBSCRIBE = MOSQ_ACL_SUBSCRIBE,
} broker_permissions_t;

typedef struct broker_binder {
    void (*on_log_print)(const char* log, broker_log_level_t level);
    int (*on_acl_check)(const char* clientid, const char* username, const char* topic, broker_permissions_t access);
    int (*on_unpwd_check)(const char* username, const char* password, int port);
    void (*on_client_connected)(const char* clientid, const char* username);
    void (*on_client_disconnected)(const char* clientid, const char* username);
    int (*on_client_do_disconnect_check)(const char* clientid);
    const char* (*get_pem_password)(unsigned int port);
} broker_binder_t;

void broker_set_binder(broker_binder_t* mosq_broker_binder);
void broker_set_running(int running);
void broker_on_log_print(const char* log, broker_log_level_t level);
int broker_on_acl_check(const char* clientid, const char* username, const char* topic, int access);
int broker_on_unpwd_check(const char* username, const char* password, int port);
void broker_on_client_connected(const char* clientid, const char* username);
void broker_on_client_disconnected(const char* clientid, const char* username);
int broker_on_client_do_disconnect_check(const char* clientid);
int broker_on_pem_password_required(char* buf, int size, int rwflag, void* userdata);
void* broker_get_pem_password_userdata(unsigned int port);
int broker_main(int argc, char* argv[]);

#endif
