#include <iostream>

#include "messenger/Messenger.h"
using namespace Messenger;

class CLog: public ILogListener {
public:
    virtual void OnTrace(const std::string& _Msg) {
        std::cout << "[T] " << _Msg << std::endl;
    }
    virtual void OnDebug(const std::string& _Msg) {
        std::cout << "[D] " << _Msg << std::endl;
    }
    virtual void OnInformation(const std::string& _Msg) {
        std::cout << "[I] " << _Msg << std::endl;
    }
    virtual void OnWarning(const std::string& _Msg) {
        std::cout << "[W] " << _Msg << std::endl;
    }
    virtual void OnError(const std::string& _Msg) {
        std::cout << "[E] " << _Msg << std::endl;
    }
};

class CEventReceiver: public IEventListener {
public:
    virtual void OnEvent(const std::string& _szSenderTag, const CEventMessage::Ptr _pEvent) {
        std::cout << "New event from: " << _szSenderTag << std::endl
                  << "    Name: " << _pEvent->GetName() << std::endl
                  << "    Params: " << _pEvent->GetParams() << std::endl;
    }

    virtual void OnLastwill(const std::string& _szSenderTag, const CLastwillMessage::Ptr _pLastwill) {
        std::cout << "New last will message from: " << _szSenderTag << std::endl
                  << "    Full message: " << _pLastwill->ToJson() << std::endl;
    }
};

// For secure connection use ssl://<IP>:8883
#define GTW_BROKER "tcp://localhost:1883"

int main() {
    CMessenger user("cli/123", "cli/123", "test", GTW_BROKER);
    CLog logger;
    CEventReceiver events;
    user.RegisterLogListener(&logger);

    try {
        user.Connect();
        // Get user
        CUser::Ptr ohm = user.GetUser("gtw/123456789/ohm");
        // Register for OHM events
        ohm->RegisterForEvents(&events);
        // Wait for events
        std::string input;
        std::cout << "Waiting events... Proceed to the request (y/n): ";
        std::cin >> input;

        if ( input=="y" ) {
            // Make a request
            CRequestMessage req;
            req.SetType("command");
            req.SetName("get_object_list_by_type");
            Poco::DynamicStruct::Ptr params = new Poco::DynamicStruct();
            params->insert("type", "controller");
            req.SetParams(params);
            // Send request to OHM
            CResponseMessage::Ptr rsp = ohm->Request(req);
            std::cout << "Response arrived: " << rsp->ToJson() << std::endl;
        }

        user.Disconnect();
    } catch (const Poco::Exception& e) {
        std::cout << "Error: " << e.message() << std::endl;
        return -1;
    }

    return 0;
}
