/****************************************************************************
 *  Copyright (c) 2018 OBLO Living LLC
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.rt-rk.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - messaging library
 * \file MqttLogger.cpp
 * \brief
 * Logger for Paho's MQTT.
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 * 13.12.2018 - Initial version
 *****************************************************************************/
#include "messenger/MqttLogger.h"

extern "C" {
#include "paho/Log.h"
}

namespace Messenger {

void LogCallback(enum LOG_LEVELS _eLevel, const char* _szMessage) {
    std::string message = _szMessage != NULL ? _szMessage : "";

    if (message.length() > 20 && _szMessage[8]==' '
            && _szMessage[15]=='.' && _szMessage[19]==' ' ) {
        // strip date and time from the begining of the message.
        // Format: '20181214 080018.933 ' = 20 characters
        message = message.substr(20);
    }

    switch (_eLevel) {
        case INVALID_LEVEL:
            break;

        case TRACE_MAXIMUM:
            break;

        case TRACE_MEDIUM:
            CMqttLogger::getInstance().OnTrace(message);
            break;

        case TRACE_MINIMUM:
            CMqttLogger::getInstance().OnDebug(message);
            break;

        case TRACE_PROTOCOL:
            CMqttLogger::getInstance().OnInformation(message);
            break;

        case LOG_ERROR:
            CMqttLogger::getInstance().OnError(message);
            break;

        case LOG_SEVERE:
            CMqttLogger::getInstance().OnWarning(message);
            break;

        case LOG_FATAL:
            CMqttLogger::getInstance().OnFatal(message);
            break;

        default:
            break;
    }
}

CMqttLogger::CMqttLogger(): m_pLogger(NULL) {
    Log_setTraceCallback(&LogCallback);
    Log_setTraceLevel(TRACE_MAXIMUM);
}

CMqttLogger::~CMqttLogger() {
}

void CMqttLogger::SetListener(ILogListener* _pLogger) {
    m_pLogger = _pLogger;
}

CMqttLogger& CMqttLogger::getInstance() {
    static CMqttLogger instance;
    return instance;
}

void CMqttLogger::OnTrace(const std::string& _Msg) {
    if (m_pLogger) {
        m_pLogger->OnTrace(_Msg);
    }
}

void CMqttLogger::OnDebug(const std::string& _Msg) {
    if (m_pLogger) {
        m_pLogger->OnDebug(_Msg);
    }
}

void CMqttLogger::OnInformation(const std::string& _Msg) {
    if (m_pLogger) {
        m_pLogger->OnInformation(_Msg);
    }
}

void CMqttLogger::OnWarning(const std::string& _Msg) {
    if (m_pLogger) {
        m_pLogger->OnWarning(_Msg);
    }
}

void CMqttLogger::OnError(const std::string& _Msg) {
    if (m_pLogger) {
        m_pLogger->OnError(_Msg);
    }
}

void CMqttLogger::OnFatal(const std::string& _Msg) {
    if (m_pLogger) {
        m_pLogger->OnError(_Msg);
    }
}

}
