#include "messenger/MqttMessage.h"
#include <cstring>

namespace Mqtt {

const MQTTClient_message& InitMsg() {
    static MQTTClient_message initMsg = MQTTClient_message_initializer;
    return initMsg;
}

CMessage::CMessage():
    m_QoS(QOS0),
    m_bRetained(false),
    m_bDuplicated(false) {
}

CMessage::CMessage(const std::string& _szPayload):
    m_szPayload(_szPayload),
    m_QoS(QOS0),
    m_bRetained(false),
    m_bDuplicated(false) {
}

CMessage::CMessage(const CMessage& _Msg):
    m_szPayload(_Msg.m_szPayload),
    m_QoS(_Msg.m_QoS),
    m_bRetained(_Msg.m_bRetained),
    m_bDuplicated(_Msg.m_bDuplicated) {
}

CMessage& CMessage::operator=(const CMessage& _Msg) {
    if (&_Msg != this) {
        m_szPayload = _Msg.m_szPayload;
        m_QoS = _Msg.m_QoS;
        m_bRetained = _Msg.m_bRetained;
        m_bDuplicated = _Msg.m_bDuplicated;
    }

    return *this;
}

CMessage::~CMessage() {}

std::string CMessage::GetPayload() const {
    return m_szPayload;
}

std::string& CMessage::GetPayloadHolder() {
    return m_szPayload;
}

void CMessage::SetPayload(const std::string& _szPayload) {
    m_szPayload = _szPayload;
}

CMessage::QosLevel CMessage::GetQos() const {
    return m_QoS;
}

void CMessage::SetQos(CMessage::QosLevel _Level) {
    m_QoS = _Level;
}

bool CMessage::IsDuplicated() const {
    return m_bDuplicated;
}

void CMessage::SetDuplicated(bool _bDuplicated) {
    m_bDuplicated = _bDuplicated;
}

bool CMessage::IsRetained() const {
    return m_bRetained;
}

void CMessage::SetRetained(bool _bRetained) {
    m_bRetained = _bRetained;
}

void CMessage::WriteMessage(MQTTClient_message** _pMsg) const {
    *_pMsg = (MQTTClient_message*) malloc(sizeof(MQTTClient_message));
    ** _pMsg = InitMsg();
    //char* payload = strdup(m_szPayload.c_str());
    char* payload = const_cast<char*>(m_szPayload.c_str());
    (*_pMsg)->payloadlen = m_szPayload.length();
    (*_pMsg)->payload = payload;
    (*_pMsg)->qos = m_QoS;
    (*_pMsg)->retained = m_bRetained ? 1 : 0;
    (*_pMsg)->dup = m_bDuplicated ? 1 : 0;
}

void CMessage::FreeMessage(MQTTClient_message** _pMsg) {
    if ( _pMsg!=NULL ) {
        MQTTClient_message* msg = *_pMsg;
        //free(msg->payload);
        free(msg);
        msg = NULL;
    }
}

CMessage::Ptr CMessage::ParseMessage(const MQTTClient_message* _pMsg) {
    CMessage::Ptr dest = new CMessage();
    std::string payload((char*)_pMsg->payload, _pMsg->payloadlen);
    dest->SetPayload(payload);
    dest->SetQos((QosLevel)_pMsg->qos);
    dest->SetRetained(_pMsg->retained);
    dest->SetDuplicated(_pMsg->dup);
    return dest;
}

}
