#include "messenger/Runners.h"

namespace Messenger {

CSynchronizedRunner::CSynchronizedRunner() {
    Reinitialize();
}

CSynchronizedRunner::~CSynchronizedRunner() {
    Stop();
}

void CSynchronizedRunner::Reinitialize() {
    m_bRunning = true;
}

void CSynchronizedRunner::Stop() {
    Lock();
    m_bRunning = false;
    Unlock();
    Continue();
}

void CSynchronizedRunner::Pause() {
    if ( IsRunning() ) {
        m_Sync.wait();
    }
}

void CSynchronizedRunner::Continue() {
    m_Sync.notify();
}

void CSynchronizedRunner::Lock() {
    m_Mutex.lock();
}

void CSynchronizedRunner::Unlock() {
    m_Mutex.unlock();
}

bool CSynchronizedRunner::IsRunning() const {
    return m_bRunning;
}

void CSynchronizedRunner::run() {
    while ( IsRunning() ) {
        DoJob();
    }
}

}
