#include "messenger/User.h"
#include "messenger/Messenger.h"

namespace Messenger {


CFunctionality::CFunctionality(const std::string& _szName): m_pUser(NULL) {
    m_szName = _szName;
}

void CFunctionality::SetOwner(CUser* _pUser) {
    m_pUser = _pUser;
}

void CFunctionality::RegisterForEvents(IEventListener* _pEventListener) {
    if ( m_pUser!=NULL ) {
        m_pUser->RegisterForEvents(_pEventListener, m_szName);
    } else {
        throw Poco::NullPointerException("Message user is not set.");
    }
}

void CFunctionality::UnregisterFromEvents() {
    if ( m_pUser!=NULL ) {
        m_pUser->UnregisterFromEvents(m_szName);
    } else {
        throw Poco::NullPointerException("Message user is not set.");
    }
}

const CResponseMessage::Ptr CFunctionality::Request(const CRequestMessage& _Request,
        unsigned int _nTimeout) {
    if ( m_pUser!=NULL ) {
        return m_pUser->Request(_Request, m_szName, _nTimeout);
    } else {
        throw Poco::NullPointerException("Message user is not set.");
    }
}

void CFunctionality::Request(const CRequestMessage& _Request, const CRequestDelegate::Ptr _pDelegate,
                             unsigned int _nTimeout) {
    if ( m_pUser!=NULL ) {
        m_pUser->Request(_Request, _pDelegate, m_szName, _nTimeout);
    } else {
        throw Poco::NullPointerException("Message user is not set.");
    }
}

CUser::CUser(const std::string& _szTag):
    m_szTag(_szTag) {
}

CUser::CUser(const CUser& _Obj) {
    m_pMessenger = _Obj.m_pMessenger;
    m_szTag = _Obj.m_szTag;
    m_pEventListener = _Obj.m_pEventListener;
}

CUser::~CUser() {
}

void CUser::SetMessenger(CMessenger* _pMessenger) {
    m_pMessenger = _pMessenger;
}

std::string CUser::GetTag() const {
    return m_szTag;
}

void CUser::RegisterForEvents(IEventListener* _pEventListener) {
    RegisterForEvents(_pEventListener, "");
}

void CUser::UnregisterFromEvents() {
    UnregisterFromEvents("");
}

void CUser::RegisterForLwm(IEventListener* _pLwmListener) {
    if ( m_pMessenger==NULL ) {
        throw Poco::IllegalStateException("Messenger pointer is not valid.");
    }

    m_pMessenger->RegisterForLwm(_pLwmListener, m_szTag);
    m_pLwmListener = _pLwmListener;
}

void CUser::UnregisterFromLwm() {
    if ( m_pMessenger==NULL ) {
        throw Poco::IllegalStateException("Messenger pointer is not valid.");
    }

    m_pMessenger->UnregisterFromLwm(m_pLwmListener, m_szTag);
}

void CUser::RegisterForEvents(IEventListener* _pEventListener, const std::string& _szFunctionality) {
    if ( m_pMessenger==NULL ) {
        throw Poco::IllegalStateException("Messenger pointer is not valid.");
    }

    m_pMessenger->RegisterForEvents(_pEventListener, m_szTag, _szFunctionality);
    m_pEventListener = _pEventListener;
}

void CUser::UnregisterFromEvents(const std::string& _szFunctionality) {
    if ( m_pMessenger==NULL ) {
        throw Poco::IllegalStateException("Messenger pointer is not valid.");
    }

    m_pMessenger->UnregisterFromEvents(m_pEventListener, m_szTag, _szFunctionality);
}

const CResponseMessage::Ptr CUser::Request(const CRequestMessage& _Request, unsigned int _nTimeout) {
    return Request(_Request, "", _nTimeout);
}

const CResponseMessage::Ptr CUser::Request(const CRequestMessage& _Request, const std::string& _szFunctionality,
        unsigned int _nTimeout) {
    if ( m_pMessenger==NULL ) {
        throw Poco::IllegalStateException("Messenger pointer is not valid.");
    }

    CSyncRequest::Ptr request = new CSyncRequest(_Request);
    m_pMessenger->Request(request, m_szTag, _szFunctionality);

    try {
        request->Wait(_nTimeout*1000);
    } catch (const Poco::TimeoutException& e) {
        m_pMessenger->CleanRequest(request->GetId());
        throw Poco::TimeoutException("Time limit exceeded.");
    }

    const CResponseMessage::Ptr response = request->GetResponse();

    if ( response.isNull() ) {
        throw Poco::RuntimeException("Response failed.");
    }

    m_pMessenger->CleanRequest(request->GetId());
    return response;
}

void CUser::Request(const CRequestMessage& _Request, const CRequestDelegate::Ptr _pDelegate,
                    unsigned int _nTimeout) {
    return Request(_Request, _pDelegate, "", _nTimeout);
}

void CUser::SendResponse(unsigned int _nRequestId, CResponseMessage::Ptr _pResponse) {
    if ( m_pMessenger==NULL ) {
        throw Poco::IllegalStateException("Messenger pointer is not valid.");
    }

    if ( _pResponse.isNull() ) {
        throw Poco::NullPointerException("Response message is NULL.");
    }

    _pResponse->SetId(_nRequestId);
    m_pMessenger->SendResponse(m_szTag, _pResponse);
}

CFunctionality::Ptr CUser::GetFunctionality(const std::string& _szFunctName) {
    CFunctionality::Ptr funct = new CFunctionality(_szFunctName);
    funct->SetOwner(this);
    return funct;
}

void CUser::Request(const CRequestMessage& _Request, const CRequestDelegate::Ptr _pDelegate, const std::string& _szFunctionality, unsigned int _nTimeout) {
    if ( m_pMessenger==NULL ) {
        throw Poco::IllegalStateException("Messenger pointer is not valid.");
    }

    if ( _pDelegate.isNull() ) {
        throw Poco::NullPointerException("Delegate argument is not valid.");
    }

    CAsyncRequest::Ptr request = new CAsyncRequest(_Request, _pDelegate);
    m_pMessenger->Request(request, m_szTag, _szFunctionality);
    m_pMessenger->ScheduleCleanup(request->GetId(), _nTimeout);
}

}
