/****************************************************************************
 *  Copyright (c) 2016 RT-RK d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.rt-rk.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - Broker application.
 * \file Database.h
 * \brief
 * Database classes.
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef OMB_DATABASE_H
#define OMB_DATABASE_H

#include "Poco/Data/Session.h"
#include "Poco/Data/SQLite/Connector.h"
#include "Poco/DynamicStruct.h"

#include "Logger.h"

namespace MessageBroker {
namespace Database {

class ITable: public Logger::CLogger {
public:
    ITable(const std::string& _szName);
    virtual ~ITable();
    std::string GetName() const;
    virtual void CreateTable()=0;
    virtual void InitTable()=0;
    virtual unsigned int UpdateTable(unsigned int _nFromVersion)=0;
protected:
    friend class CDatabase;
    Poco::Data::Session* m_pSession;
    std::string m_szName;
};

class CParamsTable: public ITable {
public:
    static const std::string NAME;
    CParamsTable();
    virtual void CreateTable();
    virtual void InitTable();
    virtual unsigned int UpdateTable(unsigned int _nFromVersion);

    std::string GetParam(std::string _szKey, const std::string& _szDefValue="");
    bool GetParamsLike(std::string _szKey, std::map<std::string, std::string> &params, bool keepSubkey = true);
    void DeleteParamsLike(const std::string &_szSubKey);
    bool GetParamBool(std::string _szKey, bool _bDefValue=false);

    void SetParam(std::string _szKey, std::string _szValue);
    void SetParam(std::string _szKey, bool _bValue);
};

class CDatabase: public Logger::CLogger {
public:
    typedef std::vector<ITable*> Tables;

    static CDatabase& GetInstance();

    template <class T>
    static T* GetTable() {
        CDatabase& database = GetInstance();
        Tables::iterator itr;

        for ( itr=database.m_Tables.begin(); itr!=database.m_Tables.end(); ++itr ) {
            ITable* table = *itr;

            if ( table->GetName()==T::NAME ) {
                return static_cast<T*>(table);
            }
        }

        throw Poco::InvalidArgumentException("Table not registered");
    }

    void RegisterTables(const Tables& _Tables);
    virtual void InitDb();
    void CheckVersion();
    void Recreate();

    ///Initialize instance of database object that will used as singlton.
    static void InitializeInstance(const string& configPath);

private:
    CDatabase(const string& _fileName);
    ~CDatabase();
    CDatabase(CDatabase const&);
    void operator=(CDatabase const&);

    bool ContainsTable(std::string _szTableName);

    static bool m_bInitialized;
    Poco::Data::Session* m_pSession;
    Tables m_Tables;
    static CDatabase* m_Instance;
    string m_szDbFile;
};

}
}
#endif
