/****************************************************************************
 *  Copyright (c) 2019 OBLO Living LLC
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.obloliving.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - Broker application.
 * \file bridge/DnsCache.h
 * \brief
 * OBLO cache for resolved domain names.
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 * 23.1.2019 Initial version
 *****************************************************************************/
#ifndef OMB_BRIDGE_DNSCACHE_H
#define OMB_BRIDGE_DNSCACHE_H

#include <string>

namespace MessageBroker {

class CDnsCache {
public:
    /**
     * @brief CDnsCache Constructor
     * @param _szServerURI Default server URI.
     */
    CDnsCache(const std::string& _szServerURI);
    /**
     * @brief SetCacheLimit Set limit reported on number of URI re-tries, before reseting to default URI.
     * @param _nLimit Number of allowed server URI errors (@see SetServerError()). Set to 0 to disable cache.
     */
    void SetCacheLimit(int _nLimit);
    /**
     * @brief Resolve Get server URI based on caching policy.
     * @return Server URI.
     * @throws Poco::DNSException with derivations.
     */
    std::string Resolve();
    /**
     * @brief SetServerVerified Set server URI verified.
     */
    void SetServerVerified();
    /**
     * @brief SetServerError Mark previously used URI with an error.
     */
    void SetServerError();

private:
    static std::string ResolveServerURI(const std::string& _szServerURI);
    static bool IsDomainName(const std::string& _szHost);

    /// Server's domain name.
    const std::string m_szDomainNameURI;
    /// Number of allowed connection failures before resetting to domain name.
    int m_nLimit;
    /// Counter of failed connections.
    int m_nCount;
    /// Address resolved for server with domain name
    std::string m_szResolvedURI;
};

} // MessageBroker

#endif
