/****************************************************************************
 *  Copyright (c) 2016 RT-RK d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.rt-rk.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - Broker application.
 * \file Globals.h
 * \brief
 * Global defines and constants.
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef OMB_GLOBALS_H
#define OMB_GLOBALS_H

#include <string>
#include <stdint.h>

#define SAFE_DELETE(p) delete p; p = NULL;
#define SAFE_DELETE_ARRAY(a) delete [] a; a = NULL;

namespace MessageBroker {

const std::string CMD_UPDATE_PROP = "update_property";
const std::string CMD_GET_PROP = "get_property";
const std::string CMD_SET_PROP = "set_property";
const std::string CMD_SERVICE_CMD = "service_cmd";

const std::string PARAM_SERVICE = "service";
const std::string PARAM_PROPERTY = "property";
const std::string PARAM_VALUE = "value";
const std::string PARAM_CMD_NAME = "cmd_name";
const std::string PARAM_PARAMS = "params";

std::string GenerateHash(int _nLength);

const uint16_t OMB_API_VERSION = 2;

namespace Config {

const std::string CONFIG_FILE = OMB_CONFIG_FILE;

}

namespace Database {

const std::string VERSION_KEY = "db.version";
const unsigned int VERSION = 5;
const std::string DB_FILE = OMB_DB_FILE;

}

namespace OmbMessenger {

const std::string MODULE_ID = "Messenger";
const unsigned int RECONNECT_PERIOD = 15;
const int USER_EXPIRATION_DAYS = 3;
const std::string ENCRYPTION_TYPE = "aes-128-ecb";
const unsigned int USER_PASSWORD_LENGTH = 16;
const unsigned int CHALLENGE_CODE_LENGTH = 10;

const std::string FUNCTIONALITY_AUTH = "auth";

const std::string REQ_TYPE_COMMAND = "command";
const std::string REQ_TYPE_QUERY = "query";
const std::string REQ_TYPE_SERVICE_CMD = "service_cmd";
const std::string REQ_NAME_ANY = "*";

}

namespace Mosquitto {

const std::string MODULE_ID = "Mosquitto";

const std::string PROJECT_ID = "wise";
const std::string ACCOUNT_ID = "0";

#define BRIDGE_USER "stratus"

const std::string CMD_ADD_CREDENTIALS = "add_creds";
const std::string CMD_REMOVE_USER = "rmv_user";

const std::string PARAM_USERNAME = "username";
const std::string PARAM_PASSWORD = "password";
const std::string PARAM_EXPIRATION = "expiration";
const std::string PARAM_NAME = "name";
const std::string PARAM_OSTYPE = "ostype";
const std::string PARAM_APPVERSION = "appversion";

const long USER_EXP_CHECK_PERIOD = 1 * 60 * 60 * 1000;

typedef enum { PERM_SUB = 1, PERM_PUB = 2 } AcPermission;

}

#ifdef USE_OMB_BRIDGE

namespace Bridge {

const std::string MODULE_ID = "Bridge";
const std::string DEFAULT_ACCOUNT = "default";
const uint64_t ADDR_CHECK_DURATION = 60 * 1000 * 1000; //microseconds

const std::string CMD_SET_ACCOUNT = "set_acc";
const std::string CMD_SUBSCRIBE = "subscribe";
const std::string CMD_UNSUBSCRIBE = "unsubscribe";

const std::string PARAM_ACCOUNT = "account";
const std::string PARAM_SVC = "svc";
const std::string PARAM_FUNCTIONALITY = "functionality";

const std::string PROP_CLOUD_ADDR = "cloud_addr";
const std::string PROP_CLOUD_STATUS = "cloud_status";

const std::string EVENT_CLOUD_STATUS_CHANGED = "cloud_status_changed";
const std::string EVENT_CLOUD_SESSION_CLEANED = "cloud_session_cleaned";

typedef enum {
    STATUS_CONNECTED_ASSIGNED = 0,
    STATUS_CONNECTED_NOT_ASSIGNED,
    STATUS_DISCONNECTED
} CloudStatus;

const std::string KEY_ACCOUNT_ID = "bridge.cloud.acc_id";
const std::string KEY_RESET_SESSION = "bridge.cloud.reset_session";

}

#endif

#ifdef USE_OMB_LINKER

namespace Linker {

const std::string MODULE_ID = "Linker";
const std::string SSDP_ST = "urn:rt-rk-com:device:OBLOLivingGateway:1";
const std::string SSDP_IP = "239.255.255.250";
const uint16_t SSDP_PORT = 1900;
const uint16_t SSDP_LISTENER_PORT = 8003;
const uint8_t REGISTRATION_CIPHER_KEY_LENGTH = 16;
const std::string ENCRYPTION_TYPE = OmbMessenger::ENCRYPTION_TYPE;
const std::string GTW_URL_FORMAT = "ssl://%s:8883";
const uint8_t REG_TIMEOUT = 15;

const std::string PROP_MAX_LINKS = "links_max";
const std::string PROP_LINKS = "links";

}

#endif

}

#endif
