/****************************************************************************
 *  Copyright (c) 2016 RT-RK d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.rt-rk.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - Broker application.
 * \file ObloMB.h
 * \brief
 * OMB apllication core.
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef OMB_APP_H
#define OMB_APP_H

#include "AppModule.h"
#include "Logger.h"
#include "PahoMqttLogger.h"

#include "Poco/Util/ServerApplication.h"
#include "Poco/Util/Application.h"
#include "Poco/Util/Option.h"
#include "Poco/Util/OptionSet.h"
#include "Poco/Util/HelpFormatter.h"
#include "Poco/Util/AbstractConfiguration.h"

namespace MessageBroker {

using Poco::Util::Application;
using Poco::Util::Option;
using Poco::Util::OptionSet;
using Poco::Util::HelpFormatter;
using Poco::Util::AbstractConfiguration;
using Poco::Util::OptionCallback;

class COmbApplication: public Poco::Util::ServerApplication, protected Logger::CLogger, private MessageBroker::CModuleManager {
public:
    COmbApplication();
    ~COmbApplication();
    virtual const char* name() const {
        return "ObloMB Application";
    }
protected:
    virtual void initialize(Poco::Util::Application& app);
    virtual void uninitialize();
    virtual void reinitialize(Poco::Util::Application& app);
    virtual int main(const std::vector<std::string>& args);
    void handleHelp(const std::string& name, const std::string& value);
    void defineOptions(OptionSet& options);

    /// Handles config path option.
    void handleConfig(const std::string& name, const std::string& value);
    ///Shows help information.
    void displayHelp();
private:
    void PrintLogHeader();
    void PrintRunTime();
    void InitDbLoggers();
    CAppModule* CreateModule(const string& _szName);
    std::string MakePrettyText(std::string _szText, char _Symbol, unsigned int _nLineSize);
    Poco::LocalDateTime m_StartTime;

    Poco::Mutex m_AppMtx;
    bool m_bInitialized;
    bool m_bHelpRequested;
    CPahoMqttLogger* m_pPahoMqttLogger;
};

}

#endif
