/****************************************************************************
 *  Copyright (c) 2016 RT-RK d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.rt-rk.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - Broker application.
 * \file SharedValues.cpp
 * \brief
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 * 13.07.2016. Initial version.
 *****************************************************************************/
#include "SharedValues.h"
#include "Globals.h"
#include "Poco/JSON/Parser.h"
#include "Globals.h"

namespace MessageBroker {
namespace SharedValues {

using namespace std;

CFactory* CFactory::m_pInstance=NULL;

CFactory::CFactory(const string &_configPath) {
    m_pConfigValues = new Poco::Util::PropertyFileConfiguration(_configPath);
}

CFactory::~CFactory() {
    m_pConfigValues->release();
}

CFactory& CFactory::GetInstance() {
    if(CFactory::m_pInstance == NULL){
        m_pInstance = new CFactory(Config::CONFIG_FILE);
    }
    return *m_pInstance;
}

void CFactory::InitializeInstance(const string& configPath) {
    if(CFactory::m_pInstance != NULL)
        throw Poco::InvalidAccessException("Somebody already initialized SharedValue factory");
    m_pInstance = new CFactory(configPath);
}

void CFactory::SetConfigValue(const string& _szKey, const string& _szValue) {
    m_pConfigValues->setString(_szKey, _szValue);
}

CHelper::CHelper(const string& _szKeyBase):
    m_szKeyBase(_szKeyBase), m_pConfigValues(NULL) {}

CHelper::CHelper(const CHelper& _Helper):
    m_szKeyBase(_Helper.m_szKeyBase),
    m_pConfigValues(_Helper.m_pConfigValues) {
}

CHelper& CHelper::operator=(const CHelper& _Helper) {
    if ( this != &_Helper ) {
        m_szKeyBase = _Helper.m_szKeyBase;
        m_pConfigValues = _Helper.m_pConfigValues;
    }

    return *this;
}

CHelper::~CHelper() {}

string CHelper::GetKeyBase() const {
    return m_szKeyBase;
}

/// *** CGateway *** ///
const string CGateway::KEY_BASE = "gtw.";

CGateway::CGateway(): CHelper(KEY_BASE) {
    m_pGtwInfo = Platform::CPlatformFactory::MakeGtwInfo();
}

string CGateway::GetId() {
    string v;

    try {
        v = m_pGtwInfo->GetID();
    } catch (const Poco::Exception& e) {
        v = m_pConfigValues->getString(m_szKeyBase + "id");
    }

    return v;
}

string CGateway::GetSecurity() {
    string v;

    try {
        v = m_pGtwInfo->GetSecurityID();
    } catch (const Poco::Exception& e) {
        v = m_pConfigValues->getString(m_szKeyBase + "sec");
    }

    return v;
}

/// *** CApp *** ///
const string CApp::KEY_BASE = "app.";

CApp::CApp(): CHelper(KEY_BASE) {
}


string CApp::GetDBFilePath() const {
    return m_pConfigValues->getString(m_szKeyBase + "db_file", Database::DB_FILE);
}

vector<string> CApp::GetEnabledModules() const {
    string s = m_pConfigValues->getString(m_szKeyBase + "enabled_modules", "[]");
    Poco::JSON::Parser parser;
    Poco::Dynamic::Var content = parser.parse(s);
    Poco::JSON::Array::Ptr jsonArr = content.extract<Poco::JSON::Array::Ptr>();
    Poco::JSON::Array::ConstIterator itr;
    vector<string> mdls;

    for (itr=jsonArr->begin(); itr!=jsonArr->end(); ++itr) {
        const Poco::Dynamic::Var& l = *itr;
        mdls.push_back(l.convert<string>());
    }

    return mdls;
}

} // SharedValues
} // MessageBroker
