/****************************************************************************
 *  Copyright (c) 2016 RT-RK d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.rt-rk.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - Broker application.
 * \file SharedValues.h
 * \brief
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 * 13.07.2016. Initial version.
 *****************************************************************************/
#ifndef OMB_SHARED_VALUES_H
#define OMB_SHARED_VALUES_H

#include "Poco/Util/PropertyFileConfiguration.h"
#include "Poco/Mutex.h"
#include "Poco/SharedPtr.h"
#include "Platform.h"

#include <stdint.h>

namespace MessageBroker {
namespace SharedValues {

using namespace std;

class CFactory;

class CHelper {
public:
    friend class CFactory;

    CHelper(const string& _szKeyBase);
    CHelper(CHelper const& _Helper);
    CHelper& operator=(CHelper const& _Helper);
    virtual ~CHelper();
    string GetKeyBase() const;

protected:
    string m_szKeyBase;
    Poco::Util::PropertyFileConfiguration* m_pConfigValues;
};

class CGateway: public CHelper {
public:
    typedef Poco::SharedPtr<CGateway> Ptr;
    static const string KEY_BASE;

    CGateway();
    string GetId();
    string GetSecurity();
private:
    Platform::IGtwInfo::Ptr m_pGtwInfo;
};

class CApp: public CHelper {
public:
    typedef Poco::SharedPtr<CApp> Ptr;
    static const string KEY_BASE;

    CApp();
    vector<string> GetEnabledModules() const;
    string GetDBFilePath() const;
};

class CFactory {
public:
    static CFactory& GetInstance();
    template <class T>
    static Poco::SharedPtr<T> GetHelper() {
        CFactory& provider = GetInstance();
        T* p = new T();
        p->m_pConfigValues = provider.m_pConfigValues;
        return p;
    }

    void SetConfigValue(const string& _szKey, const string& _szValue);
    static void InitializeInstance(const string& configPath);
private:
    CFactory(const string& _configPath);
    ~CFactory();
    CFactory(CFactory const&);
    void operator=(CFactory const&);

    Poco::Util::PropertyFileConfiguration* m_pConfigValues;
    static CFactory* m_pInstance;
};

} // SharedValues
} // MessageBroker

#endif // OMB_SHARED_VALUES_H
