/****************************************************************************
 *  Copyright (c) 2018 RT-RK d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.rt-rk.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - Broker application.
 * \file ClientFactory.cpp
 * \brief
 * Client factory.
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#include "ClientFactory.h"

namespace MessageBroker {
namespace Mosquitto {

using namespace std;

CClientFactory::CClientFactory() {
}

CClient* CClientFactory::Create(const string& _szClientId, IClientInfoProvider& _rInfoProvider) const {
    CClient* cli = NULL;
    CClientID id;
    ParseClientID(_szClientId, id);

    if (id.GetObjectType() == CClientGtwModule::TYPE_ID) {
        cli = new CClientGtwModule(id, _rInfoProvider);
    } else if (id.GetObjectType() == CClientClient::TYPE_ID) {
        cli = new CClientClient(id, _rInfoProvider);
    } else if (id.GetObjectType() == CClientBridge::TYPE_ID) {
        cli = new CClientBridge(id, _rInfoProvider);
    } else if (id.GetObjectType() == CClientUser::TYPE_ID) {
        cli = new CClientUser(id, _rInfoProvider);
    } else if (id.GetObjectType() == CClientLocalAmc::TYPE_ID) {
        cli = new CClientLocalAmc(id, _rInfoProvider);
    } else if (id.GetObjectType() == CClientWiseHub::TYPE_ID) {
        cli = new CClientWiseHub(id, _rInfoProvider);
    } else if (id.GetObjectType() == CClientWiseIPD::TYPE_ID) {
        cli = new CClientWiseIPD(id, _rInfoProvider);
    } else {
        cli = new CDummyClient(id, _rInfoProvider);
    }

    return cli;
}

void CClientFactory::ParseClientID(const string& _szClientID, CClientID& _Output) {
    // e.g. cli/3254655234534, app/omb, svc/udm, ...
    if (_szClientID.compare(0, CClientClient::TYPE_ID.length(), CClientClient::TYPE_ID) == 0) {
        _Output.GetObjectType() = CClientClient::TYPE_ID;
    } else if (_szClientID.compare(0, CClientGtwModule::TYPE_ID.length(), CClientGtwModule::TYPE_ID) == 0) {
        _Output.GetObjectType() = CClientGtwModule::TYPE_ID;
    } else if (_szClientID.compare(0, CClientBridge::TYPE_ID.length(), CClientBridge::TYPE_ID) == 0) {
        _Output.GetObjectType() = CClientBridge::TYPE_ID;
    } else if (_szClientID.compare(0, CClientLocalAmc::TYPE_ID.length(), CClientLocalAmc::TYPE_ID) == 0) {
        _Output.GetObjectType() = CClientLocalAmc::TYPE_ID;
    } else if (_szClientID.compare(0, CClientUser::TYPE_ID.length(), CClientUser::TYPE_ID) == 0) {
        _Output.GetObjectType() = CClientUser::TYPE_ID;
    } else if (_szClientID.compare(0, CClientWiseHub::TYPE_ID.length(), CClientWiseHub::TYPE_ID) == 0) {
        _Output.GetObjectType() = CClientWiseHub::TYPE_ID;
    } else if (_szClientID.compare(0, CClientWiseIPD::TYPE_ID.length(), CClientWiseIPD::TYPE_ID) == 0) {
        _Output.GetObjectType() = CClientWiseIPD::TYPE_ID;
    }

    if (_szClientID.length() > _Output.GetObjectType().length()) {
        _Output.GetObjectID() = _szClientID.substr(_Output.GetObjectType().length());
    } else {
        _Output.GetObjectID() = _Output.GetObjectType();
    }
}

}
}
