/****************************************************************************
 *  Copyright (c) 2018 RT-RK d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.rt-rk.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - Broker application.
 * \file ClientManager.h
 * \brief
 * Client manager.
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef OMB_MOSQUITTO_CLIENTMANAGER_H
#define OMB_MOSQUITTO_CLIENTMANAGER_H

#include "Clients.h"
#include <core/Logger.h>
#include <Poco/RWLock.h>
#include <map>

namespace MessageBroker {
namespace Mosquitto {

class CClientManager: private Logger::CLogger {
public:
    CClientManager(IClientInfoProvider& _rInfoProvider);
    virtual ~CClientManager();
    void AddClient(CClient::Ptr _pCli);
    void RemoveClient(const std::string& _szClientId);
    void SetStatus(const std::string& _szClientId, bool _bOnline);
    void Clear();
    bool IsAuthorizedPublish(const std::string& _szClientId, const std::string& _szTopic);
    bool IsAuthorizedSubscribe(const std::string& _szClientId, const std::string& _szTopic);
    void GetClients(const std::string& _szType, std::vector<CClient::Ptr>& _Clients);
    void GetOnlineClients(const std::string& _szType, std::vector<CClient::Ptr>& _Clients);

private:
    CClient::Ptr GetClient(const std::string& _szClientId);

    typedef std::map<std::string, CClient::Ptr> CliMap;
    typedef std::map<std::string, CliMap> Clients; // type:(id:cli)

    IClientInfoProvider& m_rInfoProvider;
    Clients m_Clients;
    Poco::RWLock m_CliMngrMutex;
};

}
}

#endif
