/****************************************************************************
 *  Copyright (c) 2016 RT-RK d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.rt-rk.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - Broker application.
 * \file mosquitto/Tables.h
 * \brief
 * Database table classes.
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef OMB_MOSQ_TABLES_H
#define OMB_MOSQ_TABLES_H

#include "core/Database.h"

namespace MessageBroker {
namespace Mosquitto {

class CCredsTable: public Database::ITable {
public:
    static const std::string NAME;
    CCredsTable();
    virtual void CreateTable();
    virtual void InitTable();
    virtual unsigned int UpdateTable(unsigned int _nFromVersion);

    void StoreCredentials(std::string _szUsername, std::string _szPassword);
    void DeleteCredentials(std::string _szUsername);
    bool CheckCredentials(std::string _szUsername, std::string _szPassword);
};

class CAccessControlTable: public Database::ITable {
public:
    static const std::string NAME;
    CAccessControlTable();
    virtual void CreateTable();
    virtual void InitTable();
    virtual unsigned int UpdateTable(unsigned int _nFromVersion);
};

class CTmpUsersTable: public Database::ITable {
public:
    static const std::string NAME;
    CTmpUsersTable();
    virtual void CreateTable();
    virtual void InitTable();
    virtual unsigned int UpdateTable(unsigned int _nFromVersion);

    void StoreTempUser(std::string _szUsername, int _nExpiration, long _nValidUntil);
    bool ContainsTempUser(std::string _szUsername);
    void DeleteTempUser(std::string _szUsername);
    long GetExpirationTime(std::string _szUsername);
    void GetExpiredUsers(std::list<std::string>& _Users, long _nTime);
};

class CUserDetails: public Database::ITable {
public:
    static const std::string NAME;
    CUserDetails();
    virtual void CreateTable();
    virtual void InitTable();
    virtual unsigned int UpdateTable(unsigned int _nFromVersion);

    void UpdateDetails(std::string _szUsername, std::string _szName, std::string _szVersion, std::string _szType);
};

} // Mosquitto
} // MessageBroker
#endif // OMB_MOSQ_TABLES_H
