#include "core/omb-messenger/Messages.h"
#include "Poco/DynamicStruct.h"
#include "Poco/SynchronizedObject.h"
#include "Poco/Random.h"
#include "core/Globals.h"

namespace MessageBroker {
namespace OmbMessenger {

namespace Event {

#ifdef USE_OMB_BRIDGE

const std::string CAssignment::NAME = "assignment";
const std::string CAssignment::ACCOUNT_ID = "account_id";

const CAssignment::Ptr CAssignment::Parse(const Poco::DynamicStruct::Ptr _pParams) {
    CAssignment::Ptr pMsg = new CAssignment();

    try {
        std::string account = (*_pParams)[ACCOUNT_ID].convert<std::string>();
        pMsg->m_szAccountId = account;
    } catch (const Poco::Exception& e) {
        throw Poco::DataFormatException("Failed to get value from the assignment message.", -1);
    }

    return pMsg;
}

std::string CAssignment::GetAccountId() const {
    return m_szAccountId;
}

#endif

}

namespace Request {

Messenger::CRequestMessage::Ptr CMsgFactory::MakeChangeCredentials(const std::string& _szUsername,
        const std::string& _szPassword, int _nExpiration) {
    Messenger::CRequestMessage::Ptr pReq = new Messenger::CRequestMessage();
    pReq->SetType(REQ_TYPE_COMMAND);
    pReq->SetName("change_credentials");
    Poco::DynamicStruct::Ptr params = new Poco::DynamicStruct();
    params->insert("username", _szUsername);
    params->insert("password", _szPassword);
    params->insert("expiration", _nExpiration);
    pReq->SetParams(params);
    return pReq;
}

Messenger::CRequestMessage::Ptr CMsgFactory::MakeRegChallenge(const std::string& _szCode) {
    Messenger::CRequestMessage::Ptr pReq = new Messenger::CRequestMessage();
    pReq->SetType(REQ_TYPE_COMMAND);
    pReq->SetName("reg_challenge");
    Poco::DynamicStruct::Ptr params = new Poco::DynamicStruct();
    params->insert("code", _szCode);
    pReq->SetParams(params);
    return pReq;
}

const std::string CChangePassword::TYPE = REQ_TYPE_COMMAND;
const std::string CChangePassword::NAME = "change_password";

const std::string CRegisterUser::FUNCTIONALITY = FUNCTIONALITY_AUTH;
const std::string CRegisterUser::TYPE = REQ_TYPE_COMMAND;
const std::string CRegisterUser::NAME = "register_user";
const std::string CRegisterUser::KEY_ID = "id";
const std::string CRegisterUser::KEY_GROUP = "group";
const std::string CRegisterUser::DEFAULT_GROUP = "cli";
const std::string CRegisterUser::USERNAME = "username";
const std::string CRegisterUser::APPVERSION = "appversion";
const std::string CRegisterUser::OSTYPE = "ostype";

std::string CRegisterUser::GetId() const {
    return m_szId;
}

std::string CRegisterUser::GetGroup() const {
    return m_szGroup;
}

std::string CRegisterUser::GetUserName() const {
    return m_szUserName;
}

std::string CRegisterUser::GetAppVersion() const {
    return m_szAppVersion;
}

std::string CRegisterUser::GetOsType() const {
    return m_szOsType;
}

const CRegisterUser::Ptr CRegisterUser::Parse(const Messenger::CRequestMessage::Ptr _pRequest) {
    CRegisterUser::Ptr pReq = new CRegisterUser();
    Poco::DynamicStruct::Ptr params = _pRequest->GetParams();

    try {
        pReq->m_szId = (*params)[KEY_ID].extract<std::string>();
    } catch (const Poco::Exception& e) {
        throw Poco::DataFormatException("Could not extract identifier.", -1);
    }

    pReq->m_szGroup = DEFAULT_GROUP;

    if (params->contains(KEY_GROUP)) {
        pReq->m_szGroup = (*params)[KEY_GROUP].convert<std::string>();
    }

    pReq->m_szUserName = USERNAME;

    if (params->contains(USERNAME)) {
        pReq->m_szUserName = (*params)[USERNAME].convert<std::string>();
    }

    pReq->m_szAppVersion = APPVERSION;

    if (params->contains(APPVERSION)) {
        pReq->m_szAppVersion = (*params)[APPVERSION].convert<std::string>();
    }

    pReq->m_szOsType = OSTYPE;

    if (params->contains(OSTYPE)) {
        pReq->m_szOsType = (*params)[OSTYPE].convert<std::string>();
    }

    return pReq;
}

const std::string CAddUser::TYPE = REQ_TYPE_COMMAND;
const std::string CAddUser::NAME = "add_user";

std::string CAddUser::GetId() const {
    return m_szId;
}

std::string CAddUser::GetGroup() const {
    return m_szGroup;
}

int CAddUser::GetExpiration() const {
    return m_nExpiration;
}

std::string CAddUser::GetName() const {
    return m_szName;
}

std::string CAddUser::GetVersion() const {
    return m_szVersion;
}

std::string CAddUser::GetType() const {
    return m_szType;
}

const CAddUser::Ptr CAddUser::Parse(const Messenger::CRequestMessage::Ptr _pRequest) {
    CAddUser::Ptr pReq = new CAddUser();
    Poco::DynamicStruct::Ptr params = _pRequest->GetParams();

    try {
        pReq->m_szId = (*params)["id"].extract<std::string>();
        pReq->m_szGroup = (*params)["group"].extract<std::string>();
        pReq->m_nExpiration = (*params)["expiration"].convert<int>();
        pReq->m_szName = (*params)["name"].extract<std::string>();
        pReq->m_szVersion = (*params)["version"].extract<std::string>();
        pReq->m_szType = (*params)["type"].extract<std::string>();
    } catch (const Poco::Exception& e) {
        throw Poco::DataFormatException("Could not extract arguments", -1);
    }

    return pReq;
}

const std::string CRemoveUser::TYPE = REQ_TYPE_COMMAND;
const std::string CRemoveUser::NAME = "rm_user";

std::string CRemoveUser::GetUsername() const {
    return m_szUsername;
}

const CRemoveUser::Ptr CRemoveUser::Parse(const Messenger::CRequestMessage::Ptr _pRequest) {
    CRemoveUser::Ptr pReq = new CRemoveUser();
    Poco::DynamicStruct::Ptr params = _pRequest->GetParams();

    try {
        pReq->m_szUsername = (*params)["username"].extract<std::string>();
    } catch (const Poco::Exception& e) {
        throw Poco::DataFormatException("Could not extract arguments", -1);
    }

    return pReq;
}

const std::string CGetProp::TYPE = REQ_TYPE_QUERY;
const std::string CGetProp::NAME = MessageBroker::CMD_GET_PROP;
const std::string CGetProp::KEY_SERVICE = MessageBroker::PARAM_SERVICE;
const std::string CGetProp::KEY_PROPERTY = MessageBroker::PARAM_PROPERTY;
const std::string CGetProp::KEY_VALUE = MessageBroker::PARAM_VALUE;

std::string CGetProp::GetServiceName() const {
    return m_szService;
}

std::string CGetProp::GetPropertyName() const {
    return m_szProp;
}

const CGetProp::Ptr CGetProp::Parse(const Messenger::CRequestMessage::Ptr _pRequest) {
    CGetProp::Ptr pReq = new CGetProp();
    Poco::DynamicStruct::Ptr params = _pRequest->GetParams();

    try {
        pReq->m_szService = (*params)[KEY_SERVICE].extract<std::string>();
        pReq->m_szProp = (*params)[KEY_PROPERTY].extract<std::string>();
    } catch (const Poco::Exception& e) {
        throw Poco::DataFormatException("Could not extract property params.", -1);
    }

    return pReq;
}

const std::string CSetProp::TYPE = REQ_TYPE_COMMAND;
const std::string CSetProp::NAME = MessageBroker::CMD_SET_PROP;
const std::string CSetProp::KEY_SERVICE = MessageBroker::PARAM_SERVICE;
const std::string CSetProp::KEY_PROPERTY = MessageBroker::PARAM_PROPERTY;
const std::string CSetProp::KEY_VALUE = MessageBroker::PARAM_VALUE;

std::string CSetProp::GetServiceName() const {
    return m_szService;
}

std::string CSetProp::GetPropertyName() const {
    return m_szProp;
}

Poco::Dynamic::Var CSetProp::GetPropertyValue() const {
    return m_Val;
}

const CSetProp::Ptr CSetProp::Parse(const Messenger::CRequestMessage::Ptr _pRequest) {
    CSetProp::Ptr pReq = new CSetProp();
    Poco::DynamicStruct::Ptr params = _pRequest->GetParams();

    try {
        pReq->m_szService = (*params)[KEY_SERVICE].extract<std::string>();
        pReq->m_szProp = (*params)[KEY_PROPERTY].extract<std::string>();
        pReq->m_Val = (*params)[KEY_VALUE];
    } catch (const Poco::Exception& e) {
        throw Poco::DataFormatException("Could not extract property params.", -1);
    }

    return pReq;
}

const std::string CGetInfo::TYPE = REQ_TYPE_QUERY;
const std::string CGetInfo::NAME = "get_info";

const std::string CServiceCmd::TYPE = REQ_TYPE_SERVICE_CMD;
const std::string CServiceCmd::NAME = REQ_NAME_ANY;
const std::string CServiceCmd::KEY_SERVICE = "service";

std::string CServiceCmd::GetServiceName() const {
    return m_szService;
}

const CServiceCmd::Ptr CServiceCmd::Parse(const Messenger::CRequestMessage::Ptr _pRequest) {
    CServiceCmd::Ptr pReq = new CServiceCmd();
    Poco::DynamicStruct::Ptr params = _pRequest->GetParams();

    try {
        pReq->m_szService = (*params)[KEY_SERVICE].extract<std::string>();
    } catch (const Poco::Exception& e) {
        throw Poco::DataFormatException("Could not extract service name.", -1);
    }

    return pReq;
}

const std::string CGetLoggers::TYPE = REQ_TYPE_QUERY;
const std::string CGetLoggers::NAME = "get_loggers";

const std::string CSetLoggers::TYPE = REQ_TYPE_COMMAND;
const std::string CSetLoggers::NAME = "set_loggers";

const std::string CResetLoggers::TYPE = REQ_TYPE_COMMAND;
const std::string CResetLoggers::NAME = "reset_loggers";

Poco::DynamicStruct CSetLoggers::GetLoggers() const{
    return m_Loggers;
}

const CSetLoggers::Ptr CSetLoggers::Parse(const Messenger::CRequestMessage::Ptr _pRequest){
    CSetLoggers::Ptr pReq = new CSetLoggers();
    pReq->m_Loggers = *(_pRequest->GetParams());
    return pReq;
}

#ifdef USE_TEST_API
const std::string CGenPayload::TYPE = REQ_TYPE_COMMAND;
const std::string CGenPayload::NAME = "gen_payload";
const std::string CGenPayload::KEY_SIZE = "size";

unsigned int CGenPayload::GetSize() const {
    return m_nSize;
}

const CGenPayload::Ptr CGenPayload::Parse(const Messenger::CRequestMessage::Ptr _pRequest) {
    CGenPayload::Ptr pReq = new CGenPayload();
    Poco::DynamicStruct::Ptr params = _pRequest->GetParams();

    try {
        pReq->m_nSize = (*params)[KEY_SIZE].convert<int>();
    } catch (const Poco::Exception& e) {
        throw Poco::DataFormatException("Could not extract property params.", -1);
    }

    return pReq;
}

#endif

}

namespace Response {

const std::string CRegChallenge::KEY_CODE = "code";

std::string CRegChallenge::GetCode() const {
    return m_szCode;
}

const CRegChallenge::Ptr CRegChallenge::Parse(const Messenger::CResponseMessage& _Response) {
    CRegChallenge::Ptr pRsp = new CRegChallenge();
    Poco::DynamicStruct::Ptr params = _Response.GetParams();

    try {
        pRsp->m_szCode = (*params)[KEY_CODE].extract<std::string>();
    } catch (const Poco::Exception& e) {
        throw Poco::DataFormatException("Could not extract code.", -1);
    }

    return pRsp;
}

}

}
}
