/****************************************************************************
 *  Copyright (c) 2016 RT-RK d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.rt-rk.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - Broker application.
 * \file Messages.h
 * \brief
 * Message classes.
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef OMB_MESSAGES_H
#define OMB_MESSAGES_H

#include "messenger/Messages.h"

namespace MessageBroker {
namespace OmbMessenger {

namespace Event {

#ifdef USE_OMB_BRIDGE
class CAssignment {
public:
    typedef Poco::SharedPtr<CAssignment> Ptr;

    static const std::string NAME;
    static const std::string ACCOUNT_ID;

    static const CAssignment::Ptr Parse(const Poco::DynamicStruct::Ptr _pParams);
    std::string GetAccountId() const;
private:
    std::string m_szAccountId;
};
#endif

}

namespace Request {

class CMsgFactory {
public:
    static Messenger::CRequestMessage::Ptr MakeChangeCredentials(const std::string& _szUsername,
            const std::string& _szPassword, int _nExpiration);
    static Messenger::CRequestMessage::Ptr MakeRegChallenge(const std::string& _szCode);
};

class CChangePassword {
public:
    static const std::string TYPE;
    static const std::string NAME;
};

class CRegisterUser {
public:
    static const std::string FUNCTIONALITY;
    static const std::string TYPE;
    static const std::string NAME;
    static const std::string KEY_ID;
    static const std::string KEY_GROUP;
    static const std::string DEFAULT_GROUP;
    static const std::string USERNAME;
    static const std::string APPVERSION;
    static const std::string OSTYPE;
    typedef Poco::SharedPtr<CRegisterUser> Ptr;
    std::string GetId() const;
    std::string GetGroup() const;
    std::string GetUserName() const;
    std::string GetAppVersion() const;
    std::string GetOsType() const;
    static const CRegisterUser::Ptr Parse(const Messenger::CRequestMessage::Ptr _pRequest);
private:
    std::string m_szId;
    std::string m_szGroup;
    std::string m_szUserName;
    std::string m_szAppVersion;
    std::string m_szOsType;
};

class CAddUser {
public:
    static const std::string TYPE;
    static const std::string NAME;

    typedef Poco::SharedPtr<CAddUser> Ptr;

    std::string GetId() const;
    std::string GetGroup() const;
    int GetExpiration() const;
    std::string GetName() const;
    std::string GetVersion() const;
    std::string GetType() const;

    static const CAddUser::Ptr Parse(const Messenger::CRequestMessage::Ptr _pRequest);
private:
    std::string m_szId;
    std::string m_szGroup;
    int m_nExpiration;
    std::string m_szName;
    std::string m_szVersion;
    std::string m_szType;
};

class CRemoveUser {
public:
    static const std::string TYPE;
    static const std::string NAME;

    typedef Poco::SharedPtr<CRemoveUser> Ptr;

    std::string GetUsername() const;

    static const CRemoveUser::Ptr Parse(const Messenger::CRequestMessage::Ptr _pRequest);
private:
    std::string m_szUsername;
};

class CGetProp {
public:
    static const std::string TYPE;
    static const std::string NAME;
    static const std::string KEY_SERVICE;
    static const std::string KEY_PROPERTY;
    static const std::string KEY_VALUE;
    typedef Poco::SharedPtr<CGetProp> Ptr;
    std::string GetServiceName() const;
    std::string GetPropertyName() const;
    static const CGetProp::Ptr Parse(const Messenger::CRequestMessage::Ptr _pRequest);
private:
    std::string m_szService;
    std::string m_szProp;
};

class CSetProp {
public:
    static const std::string TYPE;
    static const std::string NAME;
    static const std::string KEY_SERVICE;
    static const std::string KEY_PROPERTY;
    static const std::string KEY_VALUE;
    typedef Poco::SharedPtr<CSetProp> Ptr;
    std::string GetServiceName() const;
    std::string GetPropertyName() const;
    Poco::Dynamic::Var GetPropertyValue() const;
    static const CSetProp::Ptr Parse(const Messenger::CRequestMessage::Ptr _pRequest);
private:
    std::string m_szService;
    std::string m_szProp;
    Poco::Dynamic::Var m_Val;
};

class CGetInfo {
public:
    static const std::string TYPE;
    static const std::string NAME;
};

class CServiceCmd {
public:
    static const std::string TYPE;
    static const std::string NAME;
    static const std::string KEY_SERVICE;
    typedef Poco::SharedPtr<CServiceCmd> Ptr;
    std::string GetServiceName() const;
    static const CServiceCmd::Ptr Parse(const Messenger::CRequestMessage::Ptr _pRequest);
private:
    std::string m_szService;
};

class CGetLoggers{
public:
    static const std::string TYPE;
    static const std::string NAME;
};

class CSetLoggers{
public:
    static const std::string TYPE;
    static const std::string NAME;
    typedef Poco::SharedPtr<CSetLoggers> Ptr;
    Poco::DynamicStruct GetLoggers()const;
    static const CSetLoggers::Ptr Parse(const Messenger::CRequestMessage::Ptr _pRequest);
private:
    Poco::DynamicStruct m_Loggers;
};

class CResetLoggers{
public:
    static const std::string TYPE;
    static const std::string NAME;
};

#ifdef USE_TEST_API
class CGenPayload {
public:
    CGenPayload(): m_nSize(0) {}
    static const std::string TYPE;
    static const std::string NAME;
    static const std::string KEY_SIZE;
    typedef Poco::SharedPtr<CGenPayload> Ptr;
    unsigned int GetSize() const;
    static const CGenPayload::Ptr Parse(const Messenger::CRequestMessage::Ptr _pRequest);
private:
    unsigned int m_nSize;
};
#endif

}

namespace Response {

class CRegChallenge {
public:
    static const std::string KEY_CODE;
    typedef Poco::SharedPtr<CRegChallenge> Ptr;
    std::string GetCode() const;
    static const CRegChallenge::Ptr Parse(const Messenger::CResponseMessage& _Response);
private:
    std::string m_szCode;
};

}

}
}

#endif
