/****************************************************************************
 *  Copyright (c) 2016 RT-RK d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.rt-rk.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - Broker application.
 * \file Opertions.h
 * \brief
 * OMB messaging interface operation implementation.
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef OMB_OPERATIONS_H
#define OMB_OPERATIONS_H

#include "core/AppModule.h"

#include "messenger/Messages.h"
#include "messenger/Logging.h"
#include "messenger/User.h"

#include "Poco/Random.h"

namespace MessageBroker {
namespace OmbMessenger {
namespace Async {

class CManager;

class COperation: private Messenger::ILogListener {
public:
    COperation(): m_pMsgr(NULL), m_pLog(NULL) {}
    virtual ~COperation() {}
    virtual void Process()=0;
protected:
    friend class CManager;
    void SetContext(CManager* _pMsgr);
    void SetLogListener(Messenger::ILogListener* _pLog);

    virtual void OnTrace(const std::string& _Msg);
    virtual void OnDebug(const std::string& _Msg);
    virtual void OnInformation(const std::string& _Msg);
    virtual void OnWarning(const std::string& _Msg);
    virtual void OnError(const std::string& _Msg);

    CManager* m_pMsgr;
    Messenger::ILogListener* m_pLog;
    static Poco::Random m_RandomGenerator;
};

class CManager:
    private Messenger::IQueueProcessor<Async::COperation*> {
public:
    CManager(Messenger::ILogListener* _pLog);
    virtual ~CManager();

    virtual void AddOperation(Async::COperation* _pOperation);

    virtual std::string DecryptString(const std::string& _szEncoded)=0;
    virtual Messenger::CUser::Ptr GetUser(const std::string& _szTag)=0;
    virtual Poco::DynamicStruct::Ptr SendOmbCommand(const CAppModule::ID& _ModuleId,
            const std::string& _szCmdName, const Poco::DynamicStruct& _Params)=0;
protected:
    virtual void OnProcess(Async::COperation* _pOperation);
private:
    Poco::Thread m_Thread;
    Messenger::CQueueRunner<Async::COperation*> m_AsyncOperator;
    Messenger::ILogListener* m_pLog;
};

class CChangePassword: public COperation {
public:
    CChangePassword(const std::string& _szSender,
                    const std::string& _szUsername,
                    const std::string& _szName,
                    const std::string& _szAppVersion,
                    const std::string& _szOsType,
                    int _nExpiration=0);
protected:
    virtual void Process();
    virtual void OnSuccess(unsigned int _nRequestId, const Messenger::CResponseMessage& _Response);
    virtual void OnFailure(unsigned int _nRequestId);
protected:
    std::string m_szSender;
    std::string m_szUsername;
    std::string m_szPassword;
    std::string m_szName;
    std::string m_szAppVersion;
    std::string m_szOsType;
    int m_nExpiration;
};

class CRegUser: public COperation {
public:
    CRegUser(const std::string& _szSender, const std::string& _szId, const std::string& _szGroup, const std::string& _szusername, const std::string& _szappversion, const std::string& _szostype);
    int GetRegNumber() const;
protected:
    virtual void Process();
    virtual void OnSuccess(unsigned int _nRequestId, const Messenger::CResponseMessage& _Response);
    virtual void OnFailure(unsigned int _nRequestId);
private:
    std::string m_szSender;
    std::string m_szId;
    std::string m_szGroup;
    std::string m_szUserName;
    std::string m_szAppVersion;
    std::string m_szOsType;
    int m_nRegNum;
    std::string m_szChallenge;
};

class CFinalizeUserReg: public CChangePassword {
public:
    CFinalizeUserReg(const std::string& _szSender,
                     const std::string& _szUsername,
                     int _nExpiration,
                     int _nRegNum,
                     const std::string& _szName,
                     const std::string& _szAppVersion,
                     const std::string& _szOsType);
protected:
    virtual void OnSuccess(unsigned int _nRequestId, const Messenger::CResponseMessage& _Response);
private:
    int m_nRegNum;
};

}
}
}

#endif
