/****************************************************************************
 *  Copyright (c) 2018 OBLO Living LLC
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.obloliving.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - Broker application.
 * \file bridge/AsyncActions.h
 * \brief
 * OBLO bridge async actions.
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 * 28.8.2018 moved from Bridge.h
 *****************************************************************************/
#ifndef OMB_BRIDGE_ASYNCACTIONS_H
#define OMB_BRIDGE_ASYNCACTIONS_H

#include <libs/oblomessenger/include/messenger/Runners.h>
#include <Poco/Timestamp.h>
#include <Poco/Thread.h>

namespace MessageBroker {
namespace Bridge {

class CBridge;

namespace Async {

class CAsyncFunc {
public:
    typedef void (CBridge::*AsyncFunctionPtr)(void* _pParams);

    CAsyncFunc(const AsyncFunctionPtr _pFunc, void* _pParams = NULL);
    virtual ~CAsyncFunc();
private:
    friend class CAsyncFuncHandler;
    AsyncFunctionPtr m_pFunc;
    void* m_pParams;
};

class CAsyncFuncHandler: private Messenger::IQueueProcessor<CAsyncFunc*> {
public:
    CAsyncFuncHandler(CBridge* _pObj);
    virtual ~CAsyncFuncHandler();
protected:
    void Run(CAsyncFunc* _pFunc);
    virtual void OnProcess(CAsyncFunc* _pFunc);
private:
    typedef Messenger::CQueueRunner<CAsyncFunc*> AsyncProcessor;
    AsyncProcessor m_AsyncProcessor;
    Poco::Thread m_AsyncThread;
    CBridge* m_pObj;
};

class CSessionCleaner {
public:
    CSessionCleaner(CBridge* _pObj): m_pObj(_pObj), m_bRunning(true),
        m_bConnected(false) {}
    void Run(const std::string& _szCloudAddr, Poco::Timestamp::TimeDiff _nTimeout = 0);
    void Cancel();
    bool IsCleaned() const;
    bool IsRunning() const;
private:
    CBridge* m_pObj;
    bool m_bRunning;
    bool m_bConnected;
};

class CUpdateCloudAddress: public Poco::Runnable {
public:
    CUpdateCloudAddress(CBridge* _pObj, const std::string& _szAddr);
    virtual void run();
    void Cancel();
    bool IsCanceled() const;
private:
    CBridge* m_pObj;
    const std::string m_szAddr;
    CSessionCleaner m_Cleaner;
};

} // Async
} // Bridge
} // MessageBroker

#endif
