/****************************************************************************
 *  Copyright (c) 2018 OBLO Living LLC
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.obloliving.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - Broker application.
 * \file Bridge.h
 * \brief
 * OBLO bridge classes.
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 * 2016 Initial version
 *****************************************************************************/
#ifndef OMB_BRIDGE_H
#define OMB_BRIDGE_H

#include "core/AppModule.h"
#include "core/Clients.h"
#include "core/SharedValues.h"
#include "AsyncActions.h"
#include "core/Globals.h"

#include "Poco/Condition.h"
#include "core/Globals.h"

namespace MessageBroker {
namespace Bridge {

class CBridge: public CAppModule,
    private Async::CAsyncFuncHandler,
    private IConnectionStatusListener {
public:
    typedef std::pair< std::string, std::string > RouteElement;
    typedef std::list< RouteElement > Routes;

    friend class Async::CSessionCleaner;
    friend class Async::CUpdateCloudAddress;

    CBridge();
    virtual ~CBridge();

    void Start();
    void Stop();

private:
    void RestartCloudConnection(const std::string& _szAccount, bool _bCleanSession);

    bool IsAssigned() const;
    bool IsConnectionUsable() const;
    bool IsUdmDefaultRequest(const std::string& _szTopic);

    void ConfigureLocalBridge();
    void ConfigureCloudBridge(const std::string& _szAccount, bool _bCleanSession);

    void ConstructRoutes(const std::string& _szAccount);

    void GetLocalSubscriptions(CClient::SubscriptionCollection& _Subscriptions) const;
    void GetCloudSubscriptions(CClient::SubscriptionCollection& _Subscriptions, const std::string& _szAccount) const;

    Poco::DynamicStruct::Ptr CmdSetGlobalAccount(const std::string& _szCmdName, const Poco::DynamicStruct& _Params);
    Poco::DynamicStruct::Ptr CmdUpdateProperty(const std::string& _szCmdName, const Poco::DynamicStruct& _Params);
    Poco::DynamicStruct::Ptr CmdGetProperty(const std::string& _szCmdName, const Poco::DynamicStruct& _Params);
    Poco::DynamicStruct::Ptr CmdSubscribe(const std::string& _szCmdName, const Poco::DynamicStruct& _Params);
    Poco::DynamicStruct::Ptr CmdUnsubscribe(const std::string& _szCmdName, const Poco::DynamicStruct& _Params);

    CloudStatus GetCloudStatus() const;

    void AsyncUpdateCloudAddr(void* _pParams);
    void OnAddrUpdateResult(const string& _szAddr, bool _bConnected);
    void ClearCloudUpdater(void* _pParams);
    void ResetAssignment(void* _pParams);
    void ResetCleanSession(void* _pParams);
    void SwitchToCleanSession(void* _pParams);

    void RedirectToCloud(const std::string& _szTopic, const Mqtt::CMessage& _Msg);
    void RedirectToLocal(const std::string& _szTopic, const Mqtt::CMessage& _Msg);

    void Queue(const std::string& _szTopic, const Mqtt::CMessage& _Msg);
    void SendMessageQueue();

    void OnConnected(const std::string& _szTag);
    void OnConnectFailed(const std::string& _szTag, int16_t _nCode, const std::string& _szReason);
    void OnConnectionLost(const std::string& _szTag, const std::string& _szReason);

    CClient* m_pLocalClient;
    CClient* m_pCloudClient;
    std::string m_szAccount;
    std::string m_szGatewayId;
    Poco::Timestamp m_tDisconnectedSince;
    Routes m_Routes;

    typedef std::queue<Mqtt::CRecvHolder::Ptr> MsgBuffer;
    MsgBuffer m_MsgQueue;
    int m_nMsgQueueMax;

    Async::CUpdateCloudAddress* m_pCloudAddrUpdater;
    Poco::Thread* m_pCloudAddrUpdaterThread;
    Poco::Mutex m_CloudUpdaterMutex;
};

} // Bridge
} // MessageBroker

#endif
