/****************************************************************************
 *  Copyright (c) 2018 OBLO Living LLC
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.obloliving.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - Broker application.
 * \file bridge/Config.h
 * \brief
 * OBLO bridge configuration.
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 * 28.8.2018 moved from Bridge.h
 *****************************************************************************/
#ifndef OMB_BRIDGE_CONFIG_H
#define OMB_BRIDGE_CONFIG_H

#include "core/SharedValues.h"
#include <string>

namespace MessageBroker {
namespace Bridge {

class CConfigLocal: public SharedValues::CHelper {
public:
    typedef Poco::SharedPtr<CConfigLocal> Ptr;
    static const std::string KEY_BASE;

    CConfigLocal();
    std::string GetId() const;
    std::string GetUsername() const;
    std::string GetPassword() const;
    std::string GetAddr() const;
    int GetReconnectPeriod() const;
    int GetKeepAliveInterval() const;
    int GetTimeoutInterval() const;
};

class CConfigCloud: public SharedValues::CHelper {
public:
    typedef Poco::SharedPtr<CConfigCloud> Ptr;
    static const std::string KEY_BASE;

    CConfigCloud();
    std::string GetId() const;
    std::string GetUsername() const;
    std::string GetPassword() const;
    std::string GetAddr() const;
    void SetAddr(const std::string& _szAddr);
    int GetReconnectPeriodMinimum() const;
    int GetReconnectPeriodMaximum() const;
    int GetKeepAliveInterval() const;
    int GetTimeoutInterval() const;
    std::string GetCaFile() const;
    int GetMsgQueueLimit() const;
    int GetSessionExpiration() const;
    int GetDnsCacheLimit() const;
    bool IsStatusForced() const;
};


} // Bridge
} // MessageBroker

#endif
