/****************************************************************************
 *  Copyright (c) 2016 RT-RK d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.rt-rk.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - Broker application.
 * \file Link.h
 * \brief
 * OMB link object implementation.
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef OMB_LINK_H
#define OMB_LINK_H

#include "core/Database.h"
#include "core/Clients.h"
#include "core/Logger.h"

namespace MessageBroker {
namespace Linker {

using namespace std;

class ILinkListener;

class CLink:
    public Logger::CLogger,
    public IConnectionStatusListener {
public:
    typedef enum { LINK_IDLE=0,
                   LINK_RUNNING,
                   LINK_OFFLINE,
                   LINK_NOT_REGISTERED,
                   LINK_ERROR,
                   LINK_DISCOVERY_STARTED,
                   LINK_DISCOVERY_COMPLETED,
                   LINK_DISCOVERY_CANCELED,
                   LINK_REG_STARTED,
                   LINK_REG_COMPLETED
                 } STATE;

    static const std::string KEY_ID;
    static const std::string KEY_STATE;

    CLink(const string& _szId, const string& _szSecId);
    virtual ~CLink();

    void Start();
    void Stop();

    string GetId() const;
    string GetSecId() const;
    string GetUrl() const;
    void SetUrl(const std::string& _szUrl);
    void SetUsername(const std::string& _szUsername);
    void SetPassword(const std::string& _szPassword);

    uint16_t GetRegFailureCnt() const;

    void SetListener(ILinkListener* _pL);
    void SetState(STATE _State);

    Poco::DynamicStruct::Ptr ToStruct() const;

private:
    void StartLocalClient();
    void StartRemoteClient();

    virtual void OnConnected(const string& _szCliId);
    virtual void OnConnectFailed(const string& _szCliId, int16_t _nCode, const string& _szReason);
    virtual void OnConnectionLost(const string& _szCliId, const string& _szCause);

    void RedirectToRemote(const string& _szTopic, const Mqtt::CMessage& _Msg);
    void RedirectToLocal(const string& _szTopic, const Mqtt::CMessage& _Msg);

    const string m_szId;
    const string m_szSecId;
    string m_szUsername;
    string m_szPassword;
    string m_szUrl;

    bool m_bRunning;
    STATE m_State;
    uint16_t m_nRegFailCnt;

    Poco::Mutex m_Mutex;

    CClient* m_pLocalClient;
    CClient* m_pRemoteClient;

    ILinkListener* m_pListener;
};

class ILinkListener {
public:
    virtual void OnStateChanged(const std::string& _szLinkId, CLink::STATE _State)=0;
};


} // Linker
} // MessageBroker

#endif
