/****************************************************************************
 *  Copyright (c) 2016 RT-RK d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.rt-rk.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - Broker application.
 * \file Linker.h
 * \brief
 * OMB module used to manage links to other OMB's.
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef OMB_LINKER_H
#define OMB_LINKER_H

#include "core/AppModule.h"
#include "feature/linker/Link.h"
#include "feature/linker/Discovery.h"
#include "feature/linker/Messages.h"
#include "feature/linker/UserRegistration.h"
#include "feature/linker/Config.h"
#include "feature/linker/Tables.h"
#include <list>

namespace MessageBroker {
namespace Linker {

using namespace std;

class CLinker;

namespace Async {

class CAsyncFunc {
public:
    typedef void (CLinker::*AsyncFunctionPtr)(void* _pParams);

    CAsyncFunc(const AsyncFunctionPtr _pFunc, void* _pParams=NULL);
    virtual ~CAsyncFunc();
private:
    friend class CAsyncFuncHandler;
    AsyncFunctionPtr m_pFunc;
    void* m_pParams;
};

class CAsyncFuncHandler: private Messenger::IQueueProcessor<CAsyncFunc*> {
public:
    CAsyncFuncHandler(CLinker* _pObj);
    virtual ~CAsyncFuncHandler();
protected:
    void Run(CAsyncFunc* _pFunc);
    virtual void OnProcess(CAsyncFunc* _pFunc);
private:
    typedef Messenger::CQueueRunner<CAsyncFunc*> AsyncProcessor;
    AsyncProcessor m_AsyncProcessor;
    Poco::Thread m_AsyncThread;
    CLinker* m_pObj;
};

} // Async

class CLinker:
    public CAppModule,
    public ILinkListener,
    public Discovery::IDeviceObserver,
    public IRegistrationListener,
    private Async::CAsyncFuncHandler,
    private CRegistrator,
    private Discovery::CCleaner {
public:
    CLinker();
    virtual ~CLinker();
    virtual void Start();
    virtual void Stop();

    virtual void GetDbTables(Database::CDatabase::Tables& _Tables) const;
private:
    void LoadLinks();
    void StartLinks();
    void ClearLinks();
    void StopLinks();

    Poco::DynamicStruct::Ptr AddLinkHandler(const std::string& _szCmdName, const Poco::DynamicStruct& _Params);

    void ChangeLinkState(const std::string& _szId, CLink::STATE _State);

    void RunDiscovery(const std::string& _szGtwId, Discovery::CResultHandler::DiscCompletedFuncPtr _pFunc, bool _bWithTimeout, void* _pData=NULL);
    virtual void OnDeviceLocated(Discovery::CGtwInfo::Ptr _pDevice);
    void CleanDiscovery(void* _pGtwId);
    virtual void OnCleanDiscovery(const std::string& _szGtwId);
    void RegistrationDiscoveryCompleted(const std::string& _szGtwId, Discovery::CGtwInfo::Ptr _pDevInfo, void* _pData);

    void Register(const std::string& _szGtwId, const std::string& _szGtwSec, const std::string& _szUrl);
    virtual void OnRegCompleted(const string& _szRemoteGtwId, const string& _szRemoteGtwSecId, const string& _szUsername,
                                const string& _szPassword, const string& _szUrl);
    virtual void OnRegFailed(const string& _szRemoteGtwId, int16_t _nCode, const string& _szCause);

    virtual void OnStateChanged(const string& _szId, CLink::STATE _State);

    void ResurectLink(void* _pParams);
    void ResurectionDiscoveryCompleted(const std::string& _szGtwId, Discovery::CGtwInfo::Ptr _pDevInfo, void* _pData);

    void ReregisterLink(void* _pParams);

    Poco::DynamicStruct::Ptr RemoveLinkHandler(const std::string& _szCmdName, const Poco::DynamicStruct& _Params);

    Poco::DynamicStruct::Ptr GetProp(const std::string& _szCmdName, const Poco::DynamicStruct& _Params);

    typedef pair<string, CLink*> LinkElm;
    typedef map<string, CLink*> Links;
    Links m_Links;

    Discovery::CManager m_DiscoveryMngr;

    typedef pair<string, Discovery::CResultHandler*> DiscReq;
    typedef std::map<string, Discovery::CResultHandler*> DiscReqs;
    DiscReqs m_DiscReqs;
    Poco::Mutex m_DiscMutex;
    CConfig::Ptr m_pConf;
    CLinksTable* m_pLinksTable;

    string m_szGtwId;
};

} // Linker
} // MessageBroker

#endif
