/****************************************************************************
 *  Copyright (c) 2016 RT-RK d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.rt-rk.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - Broker application.
 * \file Messages.cpp
 * \brief
 * OMB linker messages.
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#include "feature/linker/Messages.h"
#include "core/Globals.h"

namespace MessageBroker {
namespace Linker {
namespace Commands {

using namespace std;

const string CAddLink::NAME = "add_link";
const string CAddLink::KEY_GTW_ID = "gtw_id";
const string CAddLink::KEY_GTW_SEC = "gtw_sec";

string CAddLink::GetGtwId() const {
    return m_szGtwId;
}

string CAddLink::GetGtwSec() const {
    return m_szGtwSec;
}

CAddLink* CAddLink::Parse(const Poco::DynamicStruct& _pParams) {
    CAddLink* cmd = new CAddLink();

    try {
        cmd->m_szGtwId = _pParams[KEY_GTW_ID].extract<string>();
        cmd->m_szGtwSec = _pParams[KEY_GTW_SEC].extract<string>();
    } catch (const Poco::Exception& e) {
        throw Poco::DataFormatException("Could not extract link params.", -1);
    }

    return cmd;
}

const string CRemoveLink::NAME = "rm_link";
const string CRemoveLink::KEY_GTW_ID = "gtw_id";

string CRemoveLink::GetGtwId() const {
    return m_szGtwId;
}

CRemoveLink* CRemoveLink::Parse(const Poco::DynamicStruct& _pParams) {
    CRemoveLink* cmd = new CRemoveLink();

    try {
        cmd->m_szGtwId = _pParams[KEY_GTW_ID].extract<string>();
    } catch (const Poco::Exception& e) {
        throw Poco::DataFormatException("Could not extract link params.", -1);
    }

    return cmd;
}

} // Commands

namespace Events {

const string CLinkAdded::NAME = "link_added";
const string CLinkAdded::KEY_GTW_ID = "gtw_id";

const string CLinkRemoved::NAME = "link_removed";
const string CLinkRemoved::KEY_GTW_ID = "gtw_id";

const string CLinkAddFailed::NAME = "link_add_failed";
const string CLinkAddFailed::KEY_GTW_ID = "gtw_id";
const string CLinkAddFailed::KEY_CODE = "code";

const string CLinkStateChanged::NAME = "link_state_changed";
const string CLinkStateChanged::KEY_GTW_ID = "gtw_id";
const string CLinkStateChanged::KEY_STATE = "state";

} // Events

} // Linker
} // MessageBroker
