/****************************************************************************
 *  Copyright (c) 2016 RT-RK d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.rt-rk.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - Broker application.
 * \file Messages.h
 * \brief
 * OMB linker messages.
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef OMB_LINKER_MESSAGES_H
#define OMB_LINKER_MESSAGES_H

#include "Poco/DynamicStruct.h"
#include "core/AppModule.h"

namespace MessageBroker {
namespace Linker {
class CLinker;
namespace Commands {

class CAddLink {
public:
    static const std::string NAME;
    static const std::string KEY_GTW_ID;
    static const std::string KEY_GTW_SEC;
    typedef Poco::SharedPtr<CAddLink> Ptr;
    std::string GetGtwId() const;
    std::string GetGtwSec() const;
    static CAddLink* Parse(const Poco::DynamicStruct& _pParams);
private:
    std::string m_szGtwId;
    std::string m_szGtwSec;
};

class CRemoveLink {
public:
    static const std::string NAME;
    static const std::string KEY_GTW_ID;
    typedef Poco::SharedPtr<CRemoveLink> Ptr;
    std::string GetGtwId() const;
    static CRemoveLink* Parse(const Poco::DynamicStruct& _pParams);
private:
    std::string m_szGtwId;
};

} // Commands

namespace Events {

class CLinkAdded: public CEvent {
public:
    static const std::string NAME;
    static const std::string KEY_GTW_ID;
    CLinkAdded(const std::string& _szGtwId): CEvent(NAME), m_szGtwId(_szGtwId) {}

    virtual Poco::DynamicStruct::Ptr GetParams() const {
        Poco::DynamicStruct::Ptr p = new Poco::DynamicStruct();
        p->insert(KEY_GTW_ID, m_szGtwId);
        return p;
    }
private:
    std::string m_szGtwId;
};

class CLinkRemoved: public CEvent {
public:
    static const std::string NAME;
    static const std::string KEY_GTW_ID;
    CLinkRemoved(const std::string& _szGtwId): CEvent(NAME), m_szGtwId(_szGtwId) {}

    virtual Poco::DynamicStruct::Ptr GetParams() const {
        Poco::DynamicStruct::Ptr p = new Poco::DynamicStruct();
        p->insert(KEY_GTW_ID, m_szGtwId);
        return p;
    }
private:
    std::string m_szGtwId;
};

class CLinkAddFailed: public CEvent {
public:
    static const std::string NAME;
    static const std::string KEY_GTW_ID;
    static const std::string KEY_CODE;
    CLinkAddFailed(const std::string& _szGtwId, uint16_t _nCode): CEvent(NAME), m_szGtwId(_szGtwId), m_nCode(_nCode) {}

    virtual Poco::DynamicStruct::Ptr GetParams() const {
        Poco::DynamicStruct::Ptr p = new Poco::DynamicStruct();
        p->insert(KEY_GTW_ID, m_szGtwId);
        p->insert(KEY_CODE, m_nCode);
        return p;
    }
private:
    std::string m_szGtwId;
    uint16_t m_nCode;
};

class CLinkStateChanged: public CEvent {
public:
    static const std::string NAME;
    static const std::string KEY_GTW_ID;
    static const std::string KEY_STATE;

    CLinkStateChanged(const std::string& _szGtwId, int _nState): CEvent(NAME), m_szGtwId(_szGtwId), m_nState(_nState) {}

    virtual Poco::DynamicStruct::Ptr GetParams() const {
        Poco::DynamicStruct::Ptr p = new Poco::DynamicStruct();
        p->insert(KEY_GTW_ID, m_szGtwId);
        p->insert(KEY_STATE, m_nState);
        return p;
    }
private:
    std::string m_szGtwId;
    int m_nState;
};

} // Events
} // Linker
} // MessageBroker

#endif
