#include "feature/linker/Tables.h"
#include "core/Globals.h"
#include "Poco/Data/RecordSet.h"

namespace MessageBroker {
namespace Linker {

using namespace std;
using namespace Poco::Data::Keywords;
using Poco::Data::Session;
using Poco::Data::Statement;
using Poco::Data::RecordSet;

const std::string CLinksTable::NAME = "Links";
const std::string CLinksTable::COLUMN_GTW_ID = "GtwId";
const std::string CLinksTable::COLUMN_GTW_SEC_ID = "GtwSecId";
const std::string CLinksTable::COLUMN_USERNAME = "Username";
const std::string CLinksTable::COLUMN_PASSWORD = "Password";
const std::string CLinksTable::COLUMN_URL = "Url";

CLinksTable::CLinksTable():
    ITable(NAME) {
}

void CLinksTable::CreateTable() {
    // Create Params table
    Statement statement(*m_pSession);
    statement << "CREATE TABLE " << NAME << " ("
              << COLUMN_GTW_ID << " VARCHAR(20) NOT NULL, "
              << COLUMN_GTW_SEC_ID << " VARCHAR(20) NOT NULL, "
              << COLUMN_USERNAME << " VARCHAR(20) NOT NULL, "
              << COLUMN_PASSWORD << " VARCHAR(20) NOT NULL, "
              << COLUMN_URL << " VARCHAR(30) NOT NULL, "
              << "PRIMARY KEY (" << COLUMN_GTW_ID << "))";
    statement.execute();
    statement.reset(*m_pSession);
}

void CLinksTable::InitTable() {
}

unsigned int CLinksTable::UpdateTable(unsigned int _nFromVersion) {
    return Database::VERSION;
}

void CLinksTable::LoadLinks(std::list<CLinkRow>& _Links) {
    Statement select(*m_pSession);
    select << "SELECT * FROM " << CLinksTable::NAME;
    select.execute();
    Poco::Data::RecordSet rs(select);
    Poco::Data::RecordSet::Iterator itr = rs.begin();

    for (; itr != rs.end(); ++itr) {
        Poco::Data::Row& r(*itr);
        CLinkRow linkRow;
        linkRow.SetGtwId(r[CLinksTable::COLUMN_GTW_ID]);
        linkRow.SetGtwSecId(r[CLinksTable::COLUMN_GTW_SEC_ID]);
        linkRow.SetUsername(r[CLinksTable::COLUMN_USERNAME]);
        linkRow.SetPassword(r[CLinksTable::COLUMN_PASSWORD]);
        linkRow.SetUrl(r[CLinksTable::COLUMN_URL]);
        _Links.push_back(linkRow);
    }
}

void CLinksTable::StoreLink(std::string _szGtwId, std::string _szGtwSecId, std::string _szUsername, std::string _szPassword, std::string _szHost) {
    Statement insert(*m_pSession);
    insert << "INSERT OR REPLACE INTO " << CLinksTable::NAME << " VALUES(?, ?, ?, ?, ?)",
           use(_szGtwId),
           use(_szGtwSecId),
           use(_szUsername),
           use(_szPassword),
           use(_szHost);
    insert.execute();
}

void CLinksTable::UpdateLinkCredentials(std::string _szGtwId, std::string _szUsername, std::string _szPassword) {
    Statement update(*m_pSession);
    update << "UPDATE " << CLinksTable::NAME
           << " SET " << CLinksTable::COLUMN_USERNAME << "=?"
           << " " << CLinksTable::COLUMN_PASSWORD << "=?"
           << " WHERE " << CLinksTable::COLUMN_GTW_ID << "=?",
           use(_szUsername),
           use(_szPassword),
           use(_szGtwId);
    update.execute();
}

void CLinksTable::UpdateLinkUrl(std::string _szGtwId, std::string _szUrl) {
    Statement update(*m_pSession);
    update << "UPDATE " << CLinksTable::NAME
           << " SET " << CLinksTable::COLUMN_URL << "=?"
           << " WHERE " << CLinksTable::COLUMN_GTW_ID << "=?",
           use(_szUrl),
           use(_szGtwId);
    update.execute();
}

void CLinksTable::DeleteLink(std::string _szGtwId) {
    Statement del(*m_pSession);
    del << "DELETE FROM " << CLinksTable::NAME << " WHERE " << CLinksTable::COLUMN_GTW_ID << "=?",
        use(_szGtwId);
    del.execute();
}

CLinkRow::CLinkRow() {}

CLinkRow::CLinkRow(const CLinkRow& _Row):
    m_szGtwId(_Row.m_szGtwId),
    m_szGtwSecId(_Row.m_szGtwSecId),
    m_szUsername(_Row.m_szUsername),
    m_szPassword(_Row.m_szPassword),
    m_szUrl(_Row.m_szUrl) {}

CLinkRow& CLinkRow::operator=(const CLinkRow& _Row) {
    m_szGtwId = _Row.m_szGtwId;
    m_szGtwSecId = _Row.m_szGtwSecId;
    m_szUsername = _Row.m_szUsername;
    m_szPassword = _Row.m_szPassword;
    m_szUrl = _Row.m_szUrl;
    return *this;
}

CLinkRow::~CLinkRow() {}

std::string CLinkRow::GetGtwId() const {
    return m_szGtwId;
}

void CLinkRow::SetGtwId(const std::string& _szGtwId) {
    m_szGtwId = _szGtwId;
}

std::string CLinkRow::GetSecGtwId() const {
    return m_szGtwSecId;
}

void CLinkRow::SetGtwSecId(const std::string& _szGtwSecId) {
    m_szGtwSecId = _szGtwSecId;
}

std::string CLinkRow::GetUsername() const {
    return m_szUsername;
}

void CLinkRow::SetUsername(const std::string& _szUsername) {
    m_szUsername = _szUsername;
}

std::string CLinkRow::GetPassword() const {
    return m_szPassword;
}

void CLinkRow::SetPassword(const std::string& _szPassword) {
    m_szPassword = _szPassword;
}

std::string CLinkRow::GetUrl() const {
    return m_szUrl;
}

void CLinkRow::SetUrl(const std::string& _szUrl) {
    m_szUrl = _szUrl;
}


} // Linker
} // MessageBroker

