/****************************************************************************
 *  Copyright (c) 2016 RT-RK d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.rt-rk.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - Broker application.
 * \file linker/Tables.h
 * \brief
 * Database table classes.
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef OMB_LINKER_TABLES_H
#define OMB_LINKER_TABLES_H

#include "core/Database.h"

namespace MessageBroker {
namespace Linker {

class CLinkRow {
public:
    CLinkRow();
    CLinkRow(const CLinkRow& _Row);
    CLinkRow& operator=(const CLinkRow& _Row);

    virtual ~CLinkRow();

    std::string GetGtwId() const;
    void SetGtwId(const std::string& _szGtwId);
    std::string GetSecGtwId() const;
    void SetGtwSecId(const std::string& _szGtwSecId);
    std::string GetUsername() const;
    void SetUsername(const std::string& _szUsername);
    std::string GetPassword() const;
    void SetPassword(const std::string& _szPassword);
    std::string GetUrl() const;
    void SetUrl(const std::string& _szUrl);
private:
    std::string m_szGtwId;
    std::string m_szGtwSecId;
    std::string m_szUsername;
    std::string m_szPassword;
    std::string m_szUrl;
};

class CLinksTable: public Database::ITable {
public:
    static const std::string NAME;
    static const std::string COLUMN_GTW_ID;
    static const std::string COLUMN_GTW_SEC_ID;
    static const std::string COLUMN_USERNAME;
    static const std::string COLUMN_PASSWORD;
    static const std::string COLUMN_URL;

    CLinksTable();
    virtual void CreateTable();
    virtual void InitTable();
    virtual unsigned int UpdateTable(unsigned int _nFromVersion);

    void LoadLinks(std::list<CLinkRow>& _Links);
    void StoreLink(std::string _szGtwId, std::string _szGtwSecId, std::string _szUsername, std::string _szPassword, std::string _szHost);
    void UpdateLinkCredentials(std::string _szGtwId, std::string _szUsername, std::string _szPassword);
    void UpdateLinkUrl(std::string _szGtwId, std::string _szUrl);
    void DeleteLink(std::string _szGtwId);
};


} // Linker
} // MessageBroker
#endif // OMB_LINKER_TABLES_H
