/****************************************************************************
 *  Copyright (c) 2016 RT-RK d.o.o
 *
 *  All Rights Reserved
 *
 *  P R O P R I E T A R Y    &    C O N F I D E N T I A L
 *
 *  -----------------------------------------------------
 *  http://www.rt-rk.com
 *  -----------------------------------------------------
 *
 * \project Oblo Message Broker - Broker application.
 * \file UserRegistration.h
 * \brief
 * OMB linker user registration.
 *
 * @Author Milan Tucic
 *
 * \notes
 *
 * \history
 *****************************************************************************/
#ifndef OMB_LINKER_USER_REG_H
#define OMB_LINKER_USER_REG_H

#include "core/Logger.h"
#include "messenger/Messenger.h"
#include "Poco/Crypto/Cipher.h"
#include "Poco/Crypto/CipherFactory.h"
#include "Poco/Crypto/CipherKey.h"
#include "Poco/SHA1Engine.h"
#include "Poco/SynchronizedObject.h"

namespace MessageBroker {
namespace Linker {

class IRegistrationListener {
public:
    virtual void OnRegCompleted(const string& _szRemoteGtwId, const string& _szRemoteGtwSecId,
                                const string& _szUsername, const string& _szPassword, const string& _szUrl)=0;
    virtual void OnRegFailed(const string& _szRemoteGtwId, int16_t _nCode, const string& _szCause)=0;
};

class CRegistration:
    public Messenger::IResponder,
    private Logger::CLogger {
public:
    static const int8_t ERR_FAILURE;
    static const int8_t ERR_PARSING;
    static const int8_t ERR_PROTOCOL;
    static const int8_t ERR_TIMEOUT;

    CRegistration(const std::string& _szGtwId, const std::string& _szRemoteGtwId,
                  const std::string& _szRemoteGtwSecId, const std::string& _szRemoteUrl,
                  IRegistrationListener* _pRegL, uint8_t _nTimeout);
    virtual ~CRegistration();
    void SetSecureConnection(const std::string& _szCA);
    void Execute();

private:
    typedef std::pair<string, string> Credentials;
    Credentials GetCredentials(uint8_t _nTimeout=10);
    void RequestCredentials();

    virtual const Messenger::CResponseMessage::Ptr Respond(const Messenger::CRequestMessage::Ptr _pRequest);
    virtual const Messenger::CResponseMessage::Ptr FunctionalityRespond(const std::string& _szFunctionality,
            const Messenger::CRequestMessage::Ptr _pRequest);

    const Messenger::CResponseMessage::Ptr RespondOnChallenge(const Messenger::CRequestMessage::Ptr _pRequest);
    const Messenger::CResponseMessage::Ptr RespondOnChangeCreds(const Messenger::CRequestMessage::Ptr _pRequest);

    Poco::Crypto::Cipher::Ptr m_pCipher;
    const std::string m_szGtwId;
    const std::string m_szRemoteGtwId;
    const std::string m_szRemoteGtwSecId;
    const std::string m_szRemoteUrl;
    IRegistrationListener* m_pRegL;
    uint8_t m_nTimeout;
    Messenger::CMessenger m_Msgr;
    uint32_t m_nRegNumber;
    std::string m_szUsername;
    std::string m_szPassword;
    Poco::SynchronizedObject m_SyncObj;
    int8_t m_nErr;
};

class CRegistrator:
    private Messenger::IQueueProcessor<CRegistration*>  {
public:
    CRegistrator();
    virtual ~CRegistrator();
protected:
    void QueueRegistration(CRegistration* _pReg);
private:
    virtual void OnProcess(CRegistration* _pReg);

    Messenger::CQueueRunner<CRegistration*> m_Registrator;
    Poco::Thread m_Thread;
};

} // Linker
} // MessageBroker

#endif
