#include "messenger/Messenger.h"
using namespace Messenger;
#include <iostream>

#define USER_TAG "cli/test"
#define PRINTOUT(m) std::cout << m << std::endl;
//#define WITH_PRINTS

class CLogPrinter: public Messenger::ILogListener {
public:
    virtual void OnTrace(const std::string& _Msg) {
        //PRINTOUT("[T]" + _Msg);
    }
    virtual void OnDebug(const std::string& _Msg) {
        //PRINTOUT("[D]" + _Msg);
    }
    virtual void OnInformation(const std::string& _Msg) {
        //PRINTOUT("[I]" + _Msg);
    }
    virtual void OnWarning(const std::string& _Msg) {
        //PRINTOUT("[W]" + _Msg);
    }
    virtual void OnError(const std::string& _Msg) {
        //PRINTOUT("[E]" + _Msg);
    }
};

typedef std::pair<unsigned int, bool> TraceListElm;
typedef std::map<unsigned int, bool> TraceMap;
TraceMap traceMap;

CLogPrinter Logger;
CUser::Ptr HomeMngr;

#define CONTROLER_ID 27

void GetControllerPropList(CMessenger& _Msgr) {
#ifdef WITH_PRINTS
    PRINTOUT("Getting controller property list...");
#endif
    CRequestMessage req;
    req.SetType("command");
    req.SetName("get_controller_property_list");
    // set args
    Poco::DynamicStruct::Ptr args = new Poco::DynamicStruct();
    args->insert("controller_id", CONTROLER_ID);
    args->insert("service_name", "URLService");
    req.SetParams(args);

    try {
        HomeMngr->Request(req);
        traceMap[req.GetId()] = true;
    } catch (Poco::Exception e) {
        traceMap[req.GetId()] = false;
        PRINTOUT(e.message());
    }
}

void GetControllerPropValue(CMessenger& _Msgr) {
#ifdef WITH_PRINTS
    PRINTOUT("Getting controller property value...");
#endif
    CRequestMessage req;
    req.SetType("command");
    req.SetName("get_controller_property_value");
    // set args
    Poco::DynamicStruct::Ptr args = new Poco::DynamicStruct();
    args->insert("controller_id", CONTROLER_ID);
    args->insert("service_name", "URLService");
    args->insert("property_name", "url");
    req.SetParams(args);

    try {
        HomeMngr->Request(req);
        traceMap[req.GetId()] = true;
    } catch (Poco::Exception e) {
        traceMap[req.GetId()] = false;
        PRINTOUT(e.message());
    }
}

void GetControllerList(CMessenger& _Msgr) {
#ifdef WITH_PRINTS
    PRINTOUT("Getting list of controllers...");
#endif
    CRequestMessage req;
    req.SetType("command");
    req.SetName("get_controller_list");

    try {
        HomeMngr->Request(req);
        traceMap[req.GetId()] = true;
    } catch (Poco::Exception e) {
        traceMap[req.GetId()] = false;
        PRINTOUT(e.message());
    }
}

void GetControllerStatus(CMessenger& _Msgr) {
#ifdef WITH_PRINTS
    PRINTOUT("Getting controller status...");
#endif
    CRequestMessage req;
    req.SetType("command");
    req.SetName("get_controller_status");
    // set args
    Poco::DynamicStruct::Ptr args = new Poco::DynamicStruct();
    args->insert("controller_id", CONTROLER_ID);
    req.SetParams(args);

    try {
        HomeMngr->Request(req);
        traceMap[req.GetId()] = true;
    } catch (Poco::Exception e) {
        traceMap[req.GetId()] = false;
        PRINTOUT(e.message());
    }
}

void GetControllerClass(CMessenger& _Msgr) {
#ifdef WITH_PRINTS
    PRINTOUT("Getting controllers class...");
#endif
    CRequestMessage req;
    req.SetType("command");
    req.SetName("get_controller_class");
    // set args
    Poco::DynamicStruct::Ptr args = new Poco::DynamicStruct();
    args->insert("controller_id", CONTROLER_ID);
    req.SetParams(args);

    try {
        HomeMngr->Request(req);
        traceMap[req.GetId()] = true;
    } catch (Poco::Exception e) {
        traceMap[req.GetId()] = false;
        PRINTOUT(e.message());
    }
}

void GetControllerByClass(CMessenger& _Msgr) {
#ifdef WITH_PRINTS
    PRINTOUT("Getting controller by class...");
#endif
    CRequestMessage req;
    req.SetType("command");
    req.SetName("get_controllers_by_class");
    // set args
    Poco::DynamicStruct::Ptr args = new Poco::DynamicStruct();
    args->insert("class_name", "MessagingController");
    req.SetParams(args);

    try {
        HomeMngr->Request(req);
        traceMap[req.GetId()] = true;
    } catch (Poco::Exception e) {
        traceMap[req.GetId()] = false;
        PRINTOUT(e.message());
    }
}

#define LOOP_NUM 1000

int main() {
    CMessenger msgr("cli/test", "cli/test", "test");
    msgr.RegisterLogListener(&Logger);

    try {
        msgr.Connect();
    } catch (Poco::Exception e) {
        PRINTOUT(e.message());
        return 0;
    }

    HomeMngr = msgr.GetUser("app/ohm");

    Poco::Timestamp startTime;

    for (int i=0; i<LOOP_NUM; ++i) {
        GetControllerPropList(msgr);
        GetControllerPropValue(msgr);
        GetControllerList(msgr);
        GetControllerStatus(msgr);
        GetControllerClass(msgr);
        GetControllerByClass(msgr);
    }

    Poco::Timestamp::TimeDiff diffTime = startTime.elapsed();

    TraceMap::const_iterator itr;
    unsigned int nPass = 0;
    unsigned int nFail = 0;
    PRINTOUT("*****REPORT*****");

    for ( itr=traceMap.begin(); itr!=traceMap.end(); ++itr ) {
        TraceListElm elm = *itr;
        std::string szResult;
        elm.second ? szResult = "SUCESS" : szResult = "FAILURE";
        elm.second ? ++nPass : ++nFail;
        PRINTOUT("Request <" << elm.first << "> result: " << szResult);
    }

    PRINTOUT("***************");
    float size = traceMap.size();
    float time = diffTime/1000;
    PRINTOUT("Number of requests calculated: " << size);
    PRINTOUT("Number of passes: " << nPass);
    PRINTOUT("Number of failures: " << nFail);
    PRINTOUT("Time elapsed between first request and last response: " << time << "[ms]" );
    PRINTOUT("Average time per request+response: " << time/size << "[ms]");
    return 0;
}
