
package org.apache.jmeter.protocol.oblomsgr.client;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.jorphan.logging.LoggingManager;
import org.eclipse.paho.client.mqttv3.MqttException;

public class ClientStore {

	private static ClientStore mClientStore = new ClientStore();
	private Map<Long, IObloClient> mClients = new HashMap<Long, IObloClient>();
	private static final org.apache.log.Logger log = LoggingManager.getLoggerForClass();

	private ClientStore() {
	}

	public static ClientStore getInstance() {
		return mClientStore;
	}

	public synchronized Long addClient(IObloClient client) {
		Long threadId = Thread.currentThread().getId();
		mClients.put(threadId, client);
		log.info("New client added to store: " + threadId);
		return threadId;
	}

	public synchronized IObloClient getClientByThread() {
		long threadId = Thread.currentThread().getId();
		return getClientByThread(threadId);
	}

	public synchronized IObloClient getClientByThread(Long threadId) {
		IObloClient client = mClients.get(threadId);
		if (client == null) {
			log.warn("Client not registered: " + threadId);
		}
		return client;
	}

	public synchronized IObloClient getClientByTag(String tag) {
		IObloClient client = null;

		for (Entry<Long, IObloClient> e : mClients.entrySet()) {
			IObloClient c = e.getValue();
			if (tag == c.getTag()) {
				client = c;
				break;
			}
		}
		if (client == null) {
			log.warn("Client not registered: " + tag);
		}
		return client;
	}

	public synchronized void removeClientByThread() {
		Long threadId = Thread.currentThread().getId();
		removeClientByThread(threadId);
	}

	public synchronized void removeClientByThread(Long threadId) {
		mClients.remove(threadId);
		log.info("Client removed from store: " + threadId);
	}

	public synchronized void removeClientByTag(String tag) {
		for (Entry<Long, IObloClient> e : mClients.entrySet()) {
			IObloClient c = e.getValue();
			if (tag == c.getTag()) {
				removeClientByThread(e.getKey());
				break;
			}
		}
	}

	public synchronized void clearClients() throws MqttException {
		for (Entry<Long, IObloClient> e : mClients.entrySet()) {
			IObloClient cli = e.getValue();
			cli.close();
		}

		mClients.clear();
	}
}
