
package org.apache.jmeter.protocol.oblomsgr.control.gui;

import java.awt.BorderLayout;
import java.awt.Color;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.apache.jmeter.gui.util.JSyntaxTextArea;
import org.apache.jmeter.gui.util.JTextScrollPane;
import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.protocol.oblomsgr.sampler.ObloMsgEventListener;
import org.apache.jmeter.protocol.oblomsgr.utilities.Constants;
import org.apache.jmeter.samplers.gui.AbstractSamplerGui;
import org.apache.jmeter.testelement.TestElement;
import org.apache.jorphan.gui.JLabeledTextField;

public class ObloMsgEventListenerGui extends AbstractSamplerGui {
    private static final long serialVersionUID = 240L;

    private final JLabeledTextField timeoutField = new JLabeledTextField(Constants.OBLOMSGR_TIMEOUT);
    private final JLabeledTextField senderField = new JLabeledTextField(Constants.OBLOMSGR_EVT_SENDER);
    private final JLabeledTextField senderFunctField = new JLabeledTextField(Constants.OBLOMSGR_EVT_SENDER_FUNCT);
    private final JLabeledTextField nameField = new JLabeledTextField(Constants.OBLOMSGR_EVT_NAME);

    private final JLabel paramsArea = new JLabel(Constants.OBLOMSGR_PARAMS);
    private final JSyntaxTextArea paramsField = JSyntaxTextArea.getInstance(10, 50);
    private final JTextScrollPane paramsPanel = JTextScrollPane.getInstance(paramsField);

    public ObloMsgEventListenerGui() {
        init();
    }

    @Override
    public String getLabelResource() {
        return this.getClass().getSimpleName();
    }

    @Override
    public String getStaticLabel() {
        return Constants.OBLOMSGR_TITLE_EVT_LISTENER;
    }

    @Override
    public TestElement createTestElement() {
        ObloMsgEventListener sampler = new ObloMsgEventListener();
        modifyTestElement(sampler);
        return sampler;
    }

    @Override
    public void modifyTestElement(TestElement s) {
        ObloMsgEventListener sampler = (ObloMsgEventListener) s;
        this.configureTestElement(sampler);
        sampler.setTimeout(timeoutField.getText());
        sampler.setEvtSender(senderField.getText());
        sampler.setEvtSenderFunctionality(senderFunctField.getText());
        sampler.setEvtName(nameField.getText());
        sampler.setEvtExpectedParams(paramsField.getText());
    }

    private void init() {
        setLayout(new BorderLayout());
        setBorder(makeBorder());
        add(makeTitlePanel(), BorderLayout.NORTH);

        JPanel mainPanel = new VerticalPanel();
        add(mainPanel, BorderLayout.CENTER);

        JPanel optionPanel = new VerticalPanel();
        optionPanel.add(timeoutField);
        optionPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.gray),
                "Details"));
        mainPanel.add(optionPanel);

        JPanel evtPanel = new VerticalPanel();
        evtPanel.add(senderField);
        evtPanel.add(senderFunctField);
        evtPanel.add(nameField);
        evtPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.gray),
                "Event to wait for"));
        // Text input panel
        JPanel paramsInputPanel = new JPanel(new BorderLayout());
        paramsInputPanel.add(this.paramsArea, BorderLayout.NORTH);
        paramsInputPanel.add(this.paramsPanel, BorderLayout.CENTER);
        evtPanel.add(paramsInputPanel);

        mainPanel.add(evtPanel);

        timeoutField.setText(Constants.OBLOMSGR_DEFAULT_TIMEOUT);
    }

    @Override
    public void configure(TestElement el) {
        super.configure(el);
        ObloMsgEventListener sampler = (ObloMsgEventListener) el;
        timeoutField.setText(sampler.getTimeout());
        senderField.setText(sampler.getEvtSender());
        senderFunctField.setText(sampler.getEvtSenderFunctionality());
        nameField.setText(sampler.getEvtName());
        paramsField.setText(sampler.getEvtExpectedParams());
    }

    @Override
    public void clearGui() {
        super.clearGui();
    }
}
