
package org.apache.jmeter.protocol.oblomsgr.control.gui;

import java.awt.BorderLayout;
import java.awt.Color;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.apache.jmeter.gui.util.JSyntaxTextArea;
import org.apache.jmeter.gui.util.JTextScrollPane;
import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.protocol.oblomsgr.sampler.ObloMsgEventPublisher;
import org.apache.jmeter.protocol.oblomsgr.utilities.Constants;
import org.apache.jmeter.samplers.gui.AbstractSamplerGui;
import org.apache.jmeter.testelement.TestElement;
import org.apache.jorphan.gui.JLabeledTextField;

public class ObloMsgEventPublisherGui extends AbstractSamplerGui {
    private static final long serialVersionUID = 240L;

    private final JLabeledTextField nameField = new JLabeledTextField(Constants.OBLOMSGR_EVT_NAME);

    private final JLabel paramsArea = new JLabel(Constants.OBLOMSGR_PARAMS);
    private final JLabeledTextField functField = new JLabeledTextField(Constants.OBLOMSGR_EVT_FUNCT);
    private final JSyntaxTextArea paramsField = JSyntaxTextArea.getInstance(10, 50);
    private final JTextScrollPane paramsPanel = JTextScrollPane.getInstance(paramsField);


    public ObloMsgEventPublisherGui() {
        init();
    }

    @Override
    public String getLabelResource() {
        return this.getClass().getSimpleName();
    }

    @Override
    public String getStaticLabel() {
        return Constants.OBLOMSGR_TITLE_EVT_PUBLISHER;
    }

    @Override
    public TestElement createTestElement() {
        ObloMsgEventPublisher sampler = new ObloMsgEventPublisher();
        modifyTestElement(sampler);
        return sampler;
    }

    @Override
    public void modifyTestElement(TestElement s) {
        ObloMsgEventPublisher sampler = (ObloMsgEventPublisher) s;
        this.configureTestElement(sampler);
        sampler.setEvtFunctionality(functField.getText());
        sampler.setEvtName(nameField.getText());
        sampler.setEvtParams(paramsField.getText());
    }

    private void init() {
        setLayout(new BorderLayout());
        setBorder(makeBorder());
        add(makeTitlePanel(), BorderLayout.NORTH);

        JPanel mainPanel = new VerticalPanel();
        add(mainPanel, BorderLayout.CENTER);

        JPanel optionPanel = new VerticalPanel();
        optionPanel.add(functField);
        optionPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.gray),
                "Details"));
        mainPanel.add(optionPanel);

        JPanel reqPanel = new VerticalPanel();
        reqPanel.add(nameField);
        reqPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.gray),
                "Event"));

        JPanel paramsInputPanel = new JPanel(new BorderLayout());
        paramsInputPanel.add(this.paramsArea, BorderLayout.NORTH);
        paramsInputPanel.add(this.paramsPanel, BorderLayout.CENTER);
        reqPanel.add(paramsInputPanel);

        mainPanel.add(reqPanel);
    }

    @Override
    public void configure(TestElement el) {
        super.configure(el);
        ObloMsgEventPublisher sampler = (ObloMsgEventPublisher) el;
        functField.setText(sampler.getEvtFunctionality());
        nameField.setText(sampler.getEvtName());
        paramsField.setText(sampler.getEvtParams());
    }

    @Override
    public void clearGui() {
        super.clearGui();
    }
}
