
package org.apache.jmeter.protocol.oblomsgr.control.gui;

import java.awt.BorderLayout;
import java.awt.Color;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JComboBox;
import javax.swing.JLabel;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.gui.util.HorizontalPanel;
import org.apache.jmeter.protocol.oblomsgr.sampler.ObloMsgInitializer;
import org.apache.jmeter.protocol.oblomsgr.utilities.Constants;
import org.apache.jmeter.samplers.gui.AbstractSamplerGui;
import org.apache.jmeter.testelement.TestElement;
import org.apache.jorphan.gui.JLabeledPasswordField;
import org.apache.jorphan.gui.JLabeledTextField;

public class ObloMsgInitializerGui extends AbstractSamplerGui {
    private static final long serialVersionUID = 240L;

    private final JLabeledTextField tagField = new JLabeledTextField(Constants.OBLOMSGR_TAG);
    private final JLabeledTextField prefixField = new JLabeledTextField(Constants.OBLOMSGR_PREFIX);
    private final JLabeledTextField urlField = new JLabeledTextField(Constants.OBLOMSGR_OMB_URL);
    private final JLabeledTextField cliIdField = new JLabeledTextField(Constants.OBLOMSGR_CLIENT_ID);
    private final JLabeledTextField usernameField = new JLabeledTextField(Constants.OBLOMSGR_USERNAME);
    private final JLabeledTextField passwordField = new JLabeledTextField(Constants.OBLOMSGR_PASSWORD);
    private final JCheckBox autoReconnectField = new JCheckBox(Constants.OBLOMSGR_AUTORECONNECT);
    private final JCheckBox cleanSessionField = new JCheckBox(Constants.OBLOMSGR_CLEANSESSION);
    private final JLabeledTextField keepaliveField = new JLabeledTextField(Constants.OBLOMSGR_KEEPALIVE);
    private final JLabel protocolLabel = new JLabel(Constants.OBLOMSGR_PROTOCOL);
    private final JComboBox protocol = new JComboBox(Constants.OBLOMSGR_PROTOCOL_OPTIONS);

    public ObloMsgInitializerGui() {
        init();
    }

    @Override
    public String getLabelResource() {
        return this.getClass().getSimpleName();
    }

    @Override
    public String getStaticLabel() {
        return Constants.OBLOMSGR_TITLE_INITIALIZER;
    }

    @Override
    public TestElement createTestElement() {
        ObloMsgInitializer sampler = new ObloMsgInitializer();
        modifyTestElement(sampler);
        return sampler;
    }

    @Override
    public void modifyTestElement(TestElement s) {
        ObloMsgInitializer sampler = (ObloMsgInitializer) s;
        this.configureTestElement(sampler);
        sampler.setTag(tagField.getText());
        sampler.setTopicPrefix(prefixField.getText());
        sampler.setClientId(cliIdField.getText());
        sampler.setUsername(usernameField.getText());
        sampler.setPassword(passwordField.getText());
        sampler.setBrokerUrl(urlField.getText());
        sampler.setAutoReconnect(autoReconnectField.isSelected());
        sampler.setCleanSession(cleanSessionField.isSelected());
        sampler.setKeepalive(keepaliveField.getText());
        sampler.setProtocol(protocol.getSelectedIndex());
    }

    private void init() {
        setLayout(new BorderLayout());
        setBorder(makeBorder());
        add(makeTitlePanel(), BorderLayout.NORTH);
        JPanel mainPanel = new VerticalPanel();
        add(mainPanel, BorderLayout.CENTER);
        JPanel optionsPanel = new VerticalPanel();
        optionsPanel.add(tagField);
        optionsPanel.add(prefixField);
        optionsPanel.add(urlField);
        optionsPanel.add(autoReconnectField);
        optionsPanel.add(cleanSessionField);
        optionsPanel.add(keepaliveField);
        optionsPanel.add(cliIdField);
        optionsPanel.add(usernameField);
        optionsPanel.add(passwordField);
        JPanel protocolPanel = new HorizontalPanel();
        protocolPanel.add(protocolLabel);
        protocolPanel.add(protocol);
        optionsPanel.add(protocolPanel);
        optionsPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.gray),
                "Details"));
        mainPanel.add(optionsPanel);
        urlField.setText(Constants.OBLOMSGR_OMB_DEFAULT_URL);
        autoReconnectField.setSelected(Constants.OBLOMSGR_DEFAULT_AUTORECONNECT);
        cleanSessionField.setSelected(Constants.OBLOMSGR_DEFAULT_CLEANSESSION);
        keepaliveField.setText(Constants.OBLOMSGR_DEFAULT_KEEPALIVE);
        protocol.setSelectedIndex(Constants.OBLOMSGR_DEFAULT_PROTOCOL);
    }

    @Override
    public void configure(TestElement el) {
        super.configure(el);
        ObloMsgInitializer sampler = (ObloMsgInitializer) el;
        tagField.setText(sampler.getTag());
        prefixField.setText(sampler.getTopicPrefix());
        cliIdField.setText(sampler.getClientId());
        usernameField.setText(sampler.getUsername());
        passwordField.setText(sampler.getPassword());
        urlField.setText(sampler.getBrokerUrl());
        autoReconnectField.setSelected(sampler.IsAutoReconnect());
        cleanSessionField.setSelected(sampler.IsCleanSession());
        keepaliveField.setText(sampler.getKeepalive()+"");
        protocol.setSelectedIndex(sampler.getProtocol());
    }

    @Override
    public void clearGui() {
        super.clearGui();
    }
}
