
package org.apache.jmeter.protocol.oblomsgr.control.gui;

import java.awt.BorderLayout;
import java.awt.Color;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.apache.jmeter.gui.util.JSyntaxTextArea;
import org.apache.jmeter.gui.util.JTextScrollPane;
import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.protocol.oblomsgr.sampler.ObloMsgRequester;
import org.apache.jmeter.protocol.oblomsgr.utilities.Constants;
import org.apache.jmeter.samplers.gui.AbstractSamplerGui;
import org.apache.jmeter.testelement.TestElement;
import org.apache.jorphan.gui.JLabeledTextField;

public class ObloMsgRequesterGui extends AbstractSamplerGui {
    private static final long serialVersionUID = 240L;

    private final JLabeledTextField userField = new JLabeledTextField(Constants.OBLOMSGR_USER_TAG);
    private final JLabeledTextField timeoutField = new JLabeledTextField(Constants.OBLOMSGR_TIMEOUT);
    private final JLabeledTextField typeField = new JLabeledTextField(Constants.OBLOMSGR_REQ_TYPE);
    private final JLabeledTextField nameField = new JLabeledTextField(Constants.OBLOMSGR_REQ_NAME);

    private final JLabel paramsArea = new JLabel(Constants.OBLOMSGR_PARAMS);
    private final JSyntaxTextArea paramsField = JSyntaxTextArea.getInstance(10, 50);
    private final JTextScrollPane paramsPanel = JTextScrollPane.getInstance(paramsField);


    public ObloMsgRequesterGui() {
        init();
    }

    @Override
    public String getLabelResource() {
        return this.getClass().getSimpleName();
    }

    @Override
    public String getStaticLabel() {
        return Constants.OBLOMSGR_TITLE_REQUESTER;
    }

    @Override
    public TestElement createTestElement() {
        ObloMsgRequester sampler = new ObloMsgRequester();
        modifyTestElement(sampler);
        return sampler;
    }

    @Override
    public void modifyTestElement(TestElement s) {
        ObloMsgRequester sampler = (ObloMsgRequester) s;
        this.configureTestElement(sampler);
        sampler.setUser(userField.getText());
        sampler.setReqTimeout(timeoutField.getText());
        sampler.setReqType(typeField.getText());
        sampler.setReqName(nameField.getText());
        sampler.setReqParams(paramsField.getText());
    }

    private void init() {
        setLayout(new BorderLayout());
        setBorder(makeBorder());
        add(makeTitlePanel(), BorderLayout.NORTH);

        JPanel mainPanel = new VerticalPanel();
        add(mainPanel, BorderLayout.CENTER);

        JPanel optionPanel = new VerticalPanel();
        optionPanel.add(userField);
        optionPanel.add(timeoutField);
        optionPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.gray),
                "Details"));
        mainPanel.add(optionPanel);

        JPanel reqPanel = new VerticalPanel();
        reqPanel.add(nameField);
        reqPanel.add(typeField);
        reqPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.gray),
                "Request"));
        // Text input panel
        JPanel paramsInputPanel = new JPanel(new BorderLayout());
        paramsInputPanel.add(this.paramsArea, BorderLayout.NORTH);
        paramsInputPanel.add(this.paramsPanel, BorderLayout.CENTER);
        reqPanel.add(paramsInputPanel);

        mainPanel.add(reqPanel);

        timeoutField.setText(Constants.OBLOMSGR_DEFAULT_TIMEOUT);
    }

    @Override
    public void configure(TestElement el) {
        super.configure(el);
        ObloMsgRequester sampler = (ObloMsgRequester) el;
        userField.setText(sampler.getUser());
        timeoutField.setText(sampler.getReqTimeout());
        typeField.setText(sampler.getReqType());
        nameField.setText(sampler.getReqName());
        paramsField.setText(sampler.getReqParams());
    }

    @Override
    public void clearGui() {
        super.clearGui();
    }
}
