
package org.apache.jmeter.protocol.oblomsgr.control.gui;

import java.awt.BorderLayout;
import java.awt.Color;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.apache.jmeter.gui.util.JSyntaxTextArea;
import org.apache.jmeter.gui.util.JTextScrollPane;
import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.protocol.oblomsgr.sampler.ObloMsgResponder;
import org.apache.jmeter.protocol.oblomsgr.utilities.Constants;
import org.apache.jmeter.samplers.gui.AbstractSamplerGui;
import org.apache.jmeter.testelement.TestElement;
import org.apache.jorphan.gui.JLabeledTextField;

public class ObloMsgResponderGui extends AbstractSamplerGui {
    private static final long serialVersionUID = 240L;

    private final JLabeledTextField timeoutField = new JLabeledTextField(Constants.OBLOMSGR_TIMEOUT);
    private final JLabeledTextField functField = new JLabeledTextField(Constants.OBLOMSGR_REQ_FUNCT);
    private final JLabeledTextField typeField = new JLabeledTextField(Constants.OBLOMSGR_REQ_TYPE);
    private final JLabeledTextField nameField = new JLabeledTextField(Constants.OBLOMSGR_REQ_NAME);

    private final JLabel reqParamsArea = new JLabel(Constants.OBLOMSGR_PARAMS);
    private final JSyntaxTextArea reqParamsField = JSyntaxTextArea.getInstance(10, 50);
    private final JTextScrollPane reqParamsPanel = JTextScrollPane.getInstance(reqParamsField);

    private final JLabel rspParamsArea = new JLabel(Constants.OBLOMSGR_RSP_PARAMS);
    private final JSyntaxTextArea rspParamsField = JSyntaxTextArea.getInstance(10, 50);
    private final JTextScrollPane rspParamsPanel = JTextScrollPane.getInstance(rspParamsField);


    public ObloMsgResponderGui() {
        init();
    }

    @Override
    public String getLabelResource() {
        return this.getClass().getSimpleName();
    }

    @Override
    public String getStaticLabel() {
        return Constants.OBLOMSGR_TITLE_RESPONDER;
    }

    @Override
    public TestElement createTestElement() {
        ObloMsgResponder sampler = new ObloMsgResponder();
        modifyTestElement(sampler);
        return sampler;
    }

    @Override
    public void modifyTestElement(TestElement s) {
        ObloMsgResponder sampler = (ObloMsgResponder) s;
        this.configureTestElement(sampler);
        sampler.setTimeout(timeoutField.getText());
        sampler.setReqFunctionality(functField.getText());
        sampler.setReqType(typeField.getText());
        sampler.setReqName(nameField.getText());
        sampler.setReqParams(reqParamsField.getText());
        sampler.setRspParams(rspParamsField.getText());
    }

    private void init() {
        setLayout(new BorderLayout());
        setBorder(makeBorder());
        add(makeTitlePanel(), BorderLayout.NORTH);

        JPanel mainPanel = new VerticalPanel();
        add(mainPanel, BorderLayout.CENTER);

        JPanel optionPanel = new VerticalPanel();
        optionPanel.add(functField);
        optionPanel.add(timeoutField);
        optionPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.gray),
                "Details"));
        mainPanel.add(optionPanel);

        JPanel reqPanel = new VerticalPanel();
        reqPanel.add(typeField);
        reqPanel.add(nameField);

        JPanel reqParamsInputPanel = new JPanel(new BorderLayout());
        reqParamsInputPanel.add(this.reqParamsArea, BorderLayout.NORTH);
        reqParamsInputPanel.add(this.reqParamsPanel, BorderLayout.CENTER);
        reqPanel.add(reqParamsInputPanel);

        reqPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.gray),
                "Request to wait for"));
        mainPanel.add(reqPanel);


        // Text input panel
        JPanel rspParamsInputPanel = new JPanel(new BorderLayout());
        rspParamsInputPanel.add(this.rspParamsArea, BorderLayout.NORTH);
        rspParamsInputPanel.add(this.rspParamsPanel, BorderLayout.CENTER);
        JPanel rspPanel = new VerticalPanel();
        rspPanel.add(rspParamsInputPanel);
        rspPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.gray),
                "Response to send on request"));
        mainPanel.add(rspPanel);

        timeoutField.setText(Constants.OBLOMSGR_DEFAULT_TIMEOUT);
    }

    @Override
    public void configure(TestElement el) {
        super.configure(el);
        ObloMsgResponder sampler = (ObloMsgResponder) el;
        timeoutField.setText(sampler.getTimeout());
        functField.setText(sampler.getReqFunctionality());
        typeField.setText(sampler.getReqType());
        nameField.setText(sampler.getReqName());
        reqParamsField.setText(sampler.getReqParams());
        rspParamsField.setText(sampler.getRspParams());
    }

    @Override
    public void clearGui() {
        super.clearGui();
    }
}
