
package org.apache.jmeter.protocol.oblomsgr.data;

import org.json.JSONObject;
import java.util.Random;
import java.lang.Exception;

public class ObloMessage {

    private static Random rand = new Random();
    private int uid;
    private JSONObject params;
    private long timestamp;
    static final int PROTOCOL_OBLO = 0;
    static final int PROTOCOL_WISE = 1;

    public ObloMessage() {
        uid = rand.nextInt();

        if (uid < 0) {
            uid = 0 - uid;
        }
        params = new JSONObject();
        timestamp = System.currentTimeMillis();
    }

    public void setUid(int uid) {
        this.uid = uid;
    }

    public void setParams(String params) {
        if (!params.isEmpty()) {
            this.params = new JSONObject(params);
        }
    }

    public void setParams(JSONObject params) {
        this.params = params;
    }

    public int getUid() {
        return uid;
    }

    public JSONObject getParams() {
        return params;
    }

    public String getParamsString() {
        return getParams().toString(4);
    }

    public void setTimestamp(long timestamp) {
        this.timestamp = timestamp;
    }

    public long getTimestamp() {
        return timestamp;
    }

    protected void inflateJson(JSONObject json, int protocol) {
        json.put("uid", uid);
        if (params.length() != 0) {
            json.put("params", params);
        }

        if(protocol==PROTOCOL_WISE) {
            json.put("ts", timestamp);
        }
    }

    public String toString(int protocol) {
        JSONObject json = new JSONObject();

        json.put("uid", uid);
        if (params.length() != 0) {
            json.put("params", params);
        }

        if(protocol==PROTOCOL_WISE) {
            json.put("ts", timestamp);
        }

        return json.toString(4);
    }

    public static void parseJson(JSONObject jsonMsg, ObloMessage toMsg, int protocol) throws Exception {
        int uid = 0;
        JSONObject params = null;

        uid = jsonMsg.getInt("uid");

        if (jsonMsg.has("params")) {
            params = jsonMsg.getJSONObject("params");
        } else {
            params = new JSONObject();
        }

        if(protocol==PROTOCOL_WISE) {
            long ts = jsonMsg.getLong("ts");
            toMsg.setTimestamp(ts);
        }

        toMsg.setUid(uid);
        toMsg.setParams(params);
    }
}
