
package org.apache.jmeter.protocol.oblomsgr.sampler;

import org.apache.jmeter.protocol.oblomsgr.client.ClientStore;
import org.apache.jmeter.protocol.oblomsgr.client.IObloClient;
import org.apache.jmeter.protocol.oblomsgr.data.EventMessage;
import org.apache.jmeter.samplers.AbstractSampler;
import org.apache.jmeter.samplers.Entry;
import org.apache.jmeter.samplers.SampleResult;

public class ObloMsgEventPublisher extends AbstractSampler {
	private static final long serialVersionUID = 233L;

	public static final String EVT_NAME = "oblomsgr.req.name";
	public static final String EVT_PARAMS = "oblomsgr.req.params";
	public static final String EVT_FUNCTIONALITY = "oblomsgr.evt.funct";

	public ObloMsgEventPublisher() {
	}

	public String getEvtName() {
		return getPropertyAsString(EVT_NAME);
	}

	public String getEvtParams() {
		return getPropertyAsString(EVT_PARAMS);
	}

	public String getEvtFunctionality() {
		return getPropertyAsString(EVT_FUNCTIONALITY);
	}

	public void setEvtName(String name) {
		setProperty(EVT_NAME, name.trim());
	}

	public void setEvtParams(String params) {
		setProperty(EVT_PARAMS, params.trim());
	}

	public void setEvtFunctionality(String funct) {
		setProperty(EVT_FUNCTIONALITY, funct.trim());
	}

	@Override
	public SampleResult sample(Entry entry) {
		SampleResult result = new SampleResult();
		result.setSampleLabel(getName());
		result.sampleStart();

		ClientStore cliStore = ClientStore.getInstance();
		IObloClient client = cliStore.getClientByThread();

		if (client == null) {
			result.setResponseMessage("Client not active. Do login first.");
			result.setDataType(org.apache.jmeter.samplers.SampleResult.TEXT);
			result.setResponseCode("-1");
			return result;
		}

		EventMessage evt = new EventMessage();
		evt.setName(getEvtName());
		evt.setParams(getEvtParams());
		String payload = evt.toString(client.getProtocol());
		result.setRequestHeaders(payload);

		String topic;
		String funct = getEvtFunctionality();

		if (funct.isEmpty()) {
			topic = client.getTag() + "/evt";
		} else {
			topic = client.getTag() + "/" + funct + "/evt";
		}

		try {
			client.publish(topic, 1, payload);
			result.sampleEnd();
			result.setSuccessful(true);
			result.setResponseCode("" + 0);
			result.setResponseMessage("Event sent successfully.");
		} catch (Exception e) {
			result.sampleEnd();
			result.setSuccessful(false);
			result.setResponseMessage("Unable to send event: " + e.toString());
			result.setDataType(org.apache.jmeter.samplers.SampleResult.TEXT);
			result.setResponseCode("-1");
		}

		return result;
	}
}
