
package org.apache.jmeter.protocol.oblomsgr.sampler;

import org.apache.jmeter.protocol.oblomsgr.client.ClientStore;
import org.apache.jmeter.protocol.oblomsgr.client.IObloClient;
import org.apache.jmeter.protocol.oblomsgr.data.StatusMessage;
import org.apache.jmeter.samplers.AbstractSampler;
import org.apache.jmeter.samplers.Entry;
import org.apache.jmeter.samplers.SampleResult;
import org.apache.jmeter.protocol.oblomsgr.utilities.Constants;

public class ObloMsgStatusPublisher extends AbstractSampler {
	private static final long serialVersionUID = 233L;

	public static final String STS_ONLINE = "oblomsgr.sts.online";
	public static final String STS_TS = "oblomsgr.sts.ts";

	public ObloMsgStatusPublisher() {
	}

	public boolean getStatusOnline() {
		return getPropertyAsBoolean(STS_ONLINE);
	}

	public long getStatusTime() {
		return getPropertyAsLong(STS_TS);
	}

	public void setStatusOnline(boolean status) {
		setProperty(STS_ONLINE, status);
	}

	public void setStatusTime(long time) {
		setProperty(STS_TS, time);
	}

	@Override
	public SampleResult sample(Entry entry) {
		SampleResult result = new SampleResult();
		result.setSampleLabel(getName());
		result.sampleStart();

		ClientStore cliStore = ClientStore.getInstance();
		IObloClient client = cliStore.getClientByThread();

		if (client == null) {
			result.setResponseMessage("Client not active. Do login first.");
			result.setDataType(org.apache.jmeter.samplers.SampleResult.TEXT);
			result.setResponseCode("-1");
			return result;
		}

		StatusMessage sts = new StatusMessage();
		sts.setOnline(getStatusOnline());
		String payload = sts.toString(client.getProtocol());
		result.setRequestHeaders(payload);
		String topic = (client.getProtocol()==Constants.OBLOMSGR_PROTOCOL_WISE) ? "/sts" : "/lwm";

		try {
			client.publish(client.getTag() + topic, 1, payload);
			result.sampleEnd();
			result.setSuccessful(true);
			result.setResponseCode("" + 0);
			result.setResponseMessage("Event sent successfully.");
		} catch (Exception e) {
			result.sampleEnd();
			result.setSuccessful(false);
			result.setResponseMessage("Unable to send event: " + e.toString());
			result.setDataType(org.apache.jmeter.samplers.SampleResult.TEXT);
			result.setResponseCode("-1");
		}

		return result;
	}
}
